/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: cmdmode.c,v 1.4 2009/08/14 11:57:37 mickey Exp $ */

#include "vchatctl.h"

#if HAVE_LIB_READLINE
  #if HAVE_READLINE_H
    #include <readline.h>
    #include <history.h>
  #elif HAVE_READLINE_READLINE_H
    #include <readline/readline.h>
    #include <readline/history.h>
  #else
    #error "Location of readline.h is not known"
  #endif

  #define	HISTORY_FILE	".vchatctl_history"
  #define	HISTORY_MAX	50

#endif

#define	MAXARGS		64

/*
 * Externals.
 */

extern char	*prog_name;

/*
 * Interactive command mode.
 */

int interactive(void)
{
	char *prompt = "VCHATCTL> ";
	char *cmdline;
#if HAVE_LIB_READLINE
	char histfile[PATH_MAX];
	char *home = getenv("HOME");
#else
	char *s;
#endif
	char *argv[MAXARGS];
	int argc;
	char **ap;

	pqsetinteractive(1);

#if HAVE_LIB_READLINE
	using_history();
	stifle_history(HISTORY_MAX);

	/*
	 * Load history from file (if any).
	 */

	if(home && *home)
	{
		snprintf(histfile, sizeof(histfile), "%s/%s", home, HISTORY_FILE);
		read_history(histfile);
	}
	else
	{
		histfile[0] = 0;
	}

	/*
	 * Get input from user.
	 */

	for(; (cmdline = readline(prompt)); pqindex = 0)
	{
		add_history(cmdline);
#else
	for(; printf(prompt) && (cmdline = calloc(1, 1024)) && fgets(cmdline, 1024, stdin); pqindex = 0)
	{
		for(; (s = strrchr(cmdline, '\r')); *s = 0);
		for(; (s = strrchr(cmdline, '\n')); *s = 0);
#endif

		/*
		 * Split commandline into argument vector.
		 */

#if HAVE_STRSEP
		for(argc = 0, ap = argv; (*ap = strsep(&cmdline, " \t")) && argc < MAXARGS; )
#else
		for(argc = 0, ap = argv, *ap = strtok(cmdline, " \t"); *ap && argc < MAXARGS; )
#endif
		{
			if(**ap)
			{
				++ap;
				++argc;
			}
#if !HAVE_STRSEP
			*ap = strtok(NULL, " \t");
#endif
		}

		/*
		 * Skip empty commandline, otherwise convert commandname
		 * to uppercase.
		 */

		if(!argc)
		{
			free(cmdline);
			continue;
		}

		upperstr(argv[0]);

		/*
		 * Execute command.
		 */

		resetpqpath();

		if(vchatctl(argc, argv) == 0xDEAD)
		{
			/*
			 * User exit.
			 */

			free(cmdline);
			break;
		}

		/*
		 * Free commandline memory and continue input loop.
		 */

		free(cmdline);
	}

	/*
	 * Save history to file (if any).
	 */

#if HAVE_LIB_READLINE
	if(histfile[0])
		write_history(histfile);

	clear_history();
#endif

	printf("\nHave a nice day...\n");

	return 0;
}
