/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: vchatctl.h,v 1.8 2009/08/14 11:57:37 mickey Exp $ */

#ifndef __VCHATCTL_H__
#define	__VCHATCTL_H__

/*
 * Suck in config.h
 */

#if HAVE_CONFIG_H
  #include "config.h"
#endif

/*
 * Include commonly used header files
 */

#include <stdio.h>
#if HAVE_SYS_TYPES_H
  #include <sys/types.h>
#endif
#if HAVE_SYS_STAT_H
  #include <sys/stat.h>
#endif
#if STDC_HEADERS
  #include <stdlib.h>
  #include <stddef.h>
#else
  #if HAVE_STDLIB_H
    #include <stdlib.h>
  #endif
#endif
#if HAVE_STRING_H
  #if !STDC_HEADERS && HAVE_MEMORY_H
    #include <memory.h>                                                           #endif
  #endif
  #include <string.h>
#endif
#if HAVE_STRINGS_H
  #include <strings.h>
#endif
#if HAVE_INTTYPES_H
  #include <inttypes.h>
#else
  #if HAVE_STDINT_H
    #include <stdint.h>
  #endif
#endif
#if HAVE_UNISTD_H
  #include <unistd.h>
#endif

#if HAVE_STDARG_H
  #include <stdarg.h>
#else
  #if HAVE_VARARGS_H
    #include <varargs.h>
  #endif
#endif

#if HAVE_CTYPE_H
  #include <ctype.h>
#endif

#if HAVE_PATHS_H
  #include <paths.h>
#endif
#if HAVE_LIMITS_H
  #include <limits.h>
#endif

#if TIME_WITH_SYS_TIME
  #include <sys/time.h>
  #include <time.h>
#else
  #if HAVE_SYS_TIME_H
    #include <sys/time.h>
  #else
    #include <time.h>
  #endif                                                                        #endif
#endif

#if HAVE_ERRNO_H
  #include <errno.h>
#endif

#if HAVE_SYS_SOCKET_H
  #include <sys/socket.h>
#endif
#if HAVE_NETINET_IN_H
  #include <netinet/in.h>
#endif
#if HAVE_ARPA_INET_H
  #include <arpa/inet.h>
#endif
#if HAVE_NETDB_H
  #include <netdb.h>
#endif
#if HAVE_SYS_UN_H
  #include <sys/un.h>
#endif

/*
 * Include vcutil.h
 */

#include "../../vserver/include/vcutil.h"

/*
 * Include shared vchatctl protocol header file.
 */

#include "../../vserver/include/vctlproto.h"

/*
 * Define PATH_MAX if the system header files do not define it.
 */

#ifndef PATH_MAX
  #define PATH_MAX 256
#endif

/*
 * Define INADDR_NONE if the system header files do not define it.
 */

#ifndef INADDR_NONE
  #define INADDR_NONE	0xffffffff
#endif

/*
 * Define AF_LOCAL and PF_LOCAL if they are not defined.
 */

#ifndef AF_LOCAL
  #define AF_LOCAL AF_UNIX
#endif

#ifndef PF_LOCAL
  #define PF_LOCAL PF_UNIX
#endif

/*
 * Provide a macro for bzero for systems that do not have a bzero call.
 */

#if !HAVE_BZERO
  #if HAVE_MEMSET
    #define	bzero(a,b)		memset(a,0,b)
  #else
    #error "Neither bzero nor memset are available"
  #endif
#endif

/*
 * Provide a macro mapping snprintf() to sprintf() for systems
 * that do not have an snprintf() call.
 */

#if !HAVE_SNPRINTF
  #define snprintf(a, b, ...)	sprintf(a, __VA_ARGS__)
  #warning "sprintf() is being used for snprintf(). This is inherently unsafe"
#endif

/*
 * Provide the SUN_LEN() macro for those systems that do not have it.
 */

#ifndef SUN_LEN
  #define SUN_LEN(x) (strlen((x)->sun_path) + sizeof((x)->sun_family))
#endif

/*
 * Parameter ID macros.
 */

#define	PN_VCHATCTL	0x0001
#define	PN_VCHATCGICTL	0x0002
#define	PN_VCHATWCLIST	0x0003

#define	PN_ADD		0x0101
#define	PN_DELETE	0x0102
#define	PN_EXIT		0x0103
#define	PN_FLUSH	0x0104
#define	PN_HELP		0x0105
#define	PN_KILL		0x0106
#define PN_QUIT		0x0107
#define	PN_SEND		0x0108
#define	PN_SET		0x0109
#define PN_SHOW		0x010A
#define	PN_STOP		0x010B

#define PN_ACL		0x0201
#define	PN_USER		0x0202

#define	PR_CHANNEL	0x0304
#define	PR_NICK		0x0305

#define PR_ANONYMOUS	0x0406
#define PR_DNSCHKLVL	0x0407
#define PR_EMAIL	0x0408
#define	PR_FGTHRESHOLD	0x0409
#define PR_FLOODGUARD	0x040A
#define PR_REGONLY	0x040B
#define	PR_SERVERNAME	0x040C
#define PR_SOCKET	0x040D

#define PN_ANONYMOUS	0x0501
#define PN_DNSCHKLVL	0x0502
#define PN_EMAIL	0x0503
#define PN_FGTHRESHOLD	0x0504
#define PN_FLOODGUARD	0x0505
#define PN_LASTLOG	0x0506
#define PN_REGONLY	0x0507
#define	PN_SERVERNAME	0x0508
#define PN_SOCKET	0x0509
#define PN_UPTIME	0x050A
#define	PN_USERS	0x050B
#define	PN_VERSION	0x050C

#define	PR_ACLRULE	0x0601
#define PR_ADDRESS	0x0602
#define PR_PASSWORD	0x0603

#define PN_SERVER	0x0701

/*
 * Qualifiers ID macros:
 */

#define QR_SOCKET	0x0001

#define QR_NETMASK	0x0101
#define	QR_MASKBITS	0x0102
#define QR_TTL		0x0103
#define	QR_EXPIRES	0x0104
#define	QR_CHANNEL	0x0105

#define	QN_ACCEPT	0x0201
#define	QN_REJECT	0x0202
#define	QN_SILENT	0x0203
#define	QN_ALL		0x0204

#define	QN_BROADCAST	0x0301

#define QR_ADDRESS	0x0401
#define	QR_HOST		0x0402
#define QR_USER		0x0403
#define QR_RECORDS	0x0404
#define QR_SINCE	0x0405
#define QR_UNTIL	0x0406
#define QR_UNAME	0x0407
#define	QN_WIDE		0x0408

/*
 * Parameter/Qualifier handler function type.
 */

//typedef int (*PQFUNC)(int, char *[]);

/*
 * Function prototypes:
 */

/*-----------*
 | pqfuncs.c |
 *-----------*/

int	vchatctl(int argc, char *argv[]);

/*------------*
 | vchatctl.c |
 *------------*/

int	main(int argc, char *argv[]);

/*-----------*
 | cmdmode.c |
 *-----------*/

int	interactive(void);

/*--------*
 | subr.c |
 *--------*/

int	sock_open(void);
void	sock_close(void);

int	vctl_send(uint16_t type, uint16_t flags, void *data, size_t len);
int	vctl_sendmsg(uint16_t type, uint16_t flags, char *fmt, ...);
VCTL	*vctl_recvmsg(void);
void	vctl_showreply(void);

int	nick_ok(register char *nick);
void	ulist(int32_t *channel);
void	broadcast(void);
void	killuser(char *nick, int silent);
void	killchannel(int32_t channel, int silent);
void	killall(int silent);
void	anonymous(uint8_t *onoff);
void	email(uint8_t *onoff);
void	regonly(uint8_t *onoff);
void	floodguard(uint8_t *onoff);
void	fgthreshold(uint32_t *arg);
void	shut_down(void);
void	dnschklvl(uint8_t *arg);
void	servername(char *arg);
void	uptime(void);
void	acl_list(void);
void	acl_flush(void);
void	acl_delete(uint16_t rule);
void	acl_add(VCTL_ACL *vacl);
void	user_add(char *nick, char *passwd);
void	user_delete(char *nick);
void	cnmessage(int32_t channel, char *nick);
void	lastlog(VCTL_LLQUERY *query, int wide);
int	webchatlist(int argc, char *argv[], PQLINK *pql);

/*---------*
 | webui.c |
 *---------*/

int	webui(int argc, char *argv[], PQLINK *pql);
int	webui_ulist(void);
void	webui_kill(char *user);

/*---------*
 | usage.c |
 *---------*/

void	usage(char *fmt, ...);

#endif /* __VCHATCTL_H__ */
