/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: webui.c,v 1.3 2009/08/14 11:57:37 mickey Exp $ */

#include "vchatctl.h"

/*
 * Externals.
 */

extern char *prog_name;

/*
 * Simple HTML based UI for killing users.
 */

int webui(int argc, char *argv[], PQLINK *pql)
{
	char *query;

	/*
	 * Output proper HTTP headers and DTD.
	 */

	printf
	(
		"Content-type: text/html; charset=iso-8859-15\n"
		"Cache-control: no-cache\n"
		"Pragma: no-cache\n"
		"\n"
		"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\">\n"
	);

	/*
	 * Get CGI query string
	 */

	if((query = (char *)getenv("QUERY_STRING")) == NULL)
	{
		printf("<HTML>\n<HEAD><TITLE>FAILURE</TITLE></HEAD>\n");
		printf("<BODY BGCOLOR=#ff0000 TEXT=#ffffff LINK=#ffff00>\n");
		printf("No QUERY_STRING environment present - check script call\n");
		printf("</BODY>\n</HTML>\n");

		return -1;
	}

	/*
	 * Determine operation mode from query string
	 */

	if(!strncmp(query, "kill=", 5))
	{
		/*
		 * Kill user mode
		 */

		printf("<HTML>\n<HEAD><TITLE>VChat control panel: kill user</TITLE></HEAD>\n");
		printf("<BODY BGCOLOR=#cc3300 TEXT=#ffffff LINK=#0000ff>\n");
		printf("<CENTER><H2>VChat Control Panel</H2></CENTER>\n");
		printf("<HR><P><DL><DL>\n");

		query += 5;

		webui_kill(query);

		printf("</DL></DL>\n");
		printf("<HR><A HREF=\"vchatctl.cgi\">[Zur&uuml;ck zum VChat control panel]</A>\n");
		printf("</BODY>\n</HTML>\n");
	}
	else if(!strlen(query))
	{
		/*
		 * Ulist mode
		 */

		printf("<HTML>\n<HEAD>\n<TITLE>VChat control panel:</TITLE>\n");
		printf("<META HTTP-EQUIV=\"Refresh\" CONTENT=\"60\">\n</HEAD>\n");
		printf("<BODY BGCOLOR=#cc3300 TEXT=#ffffff LINK=#0000ff>\n");
		printf("<CENTER><H2>VChat Control Panel</H2></CENTER>\n");
		printf("<HR><P>\n");
		printf("Folgende Benutzer befinden sich augenblicklich im VChat (Klicken auf User entfernt diesen):\n");
		printf("<P><DL><DL>\n");

		webui_ulist();

		printf("</DL></DL>\n</BODY>\n</HTML>\n");
	}
	else
	{
		/*
		 * Garbled query string
		 */

		printf("<HTML>\n<HEAD><TITLE>ERROR: invalid query string supplied.</TITLE></HEAD>\n");
		printf("<BODY BGCOLOR=#ff0000 TEXT=#ffffff LINK=#ffff00>\n");
		printf("The query string you have supplied is invalid.\n");
		printf("</BODY>\n</HTML>\n");
	}

	fflush(stdout);

	return 0;
}

int webui_ulist(void)
{
	VCTL *vctl;
	int users = 0;;

	if(!vctl_send(VCTL_CMD_ULIST, VCTL_FLG_MSGCT, NULL, 0))
	{
		for( ; (vctl = vctl_recvmsg()) ; )
		{
			if(vctl->type == VCTL_MSG_UINFO)
			{
				VCTL_UINFO *vu = (VCTL_UINFO *)&vctl->data[0];

				printf("<li><a href=\"vchatctl.cgi?kill=%s\">%-16s %-11d %s</a></li>\n",
					vu->nick, vu->nick,
					vu->channel, vu->host);
				users++;
			}
			else if(vctl->type == VCTL_MSG_TEXT)
			{
				printf("%s<br/>\n", vctl->data);
			}
			else
			{
				printf("&lt;&lt;&lt;unknown control message, type=$%04x, len=$%04x&gt;&gt;&gt;<br/>\n",
					vctl->type, vctl->len);
			}

			if(vctl->flags & VCTL_FLG_MSGCT)
				break;
		}
	}
	else
	{
		printf("<P><BR><P><BR>ERROR sending control packet - aborted<P><BR><P><BR>\n");
		users = -1;
	}

	return users;
}

void webui_kill(char *user)
{
	if(!vctl_send(VCTL_CMD_KILL, VCTL_FLG_MSGCT, NULL, 0))
	{
		VCTL *vctl;

		for( ; (vctl = vctl_recvmsg()); )
		{
			if(vctl->type == VCTL_MSG_TEXT)
			{
				printf("%s<br/>\n", vctl->data);
			}
			else
			{
				printf("&lt;&lt;&lt;unknown control message, type=$%04x, len=$%04x&gt;&gt;&gt;<br/>\n",
					vctl->type, vctl->len);
			}

			if(vctl->flags & VCTL_FLG_MSGCT)
				break;
		}
	}
	else
	{
		printf("&lt;&lt;&lt;failed to send control command (%s)&gt;&gt;&gt;<br/>\n",
			strerror(errno));
	}
}
