/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: cmd_acl_add.c,v 1.13 2009/08/14 11:54:49 mickey Exp $ */

#include "vserver.h"

/*
 * Process ACL commands
 */

void cmd_acl_add(CLDATA *ptr, void *data, size_t len)
{
	VACL		vacl;

	/*
	 * Check for correct argument length depending on client proto rev.
	 */

	if(ptr->protocol >= 0x00040011 && len == sizeof(VACL))
	{
		/*
		 * Protocol 4-11 and above use 16 byte VACL struct with TTL.
		 */

		bcopy(data, &vacl, sizeof(VACL));
	}
	else if(ptr->protocol < 0x00040011 && len == 12)
	{
		/*
		 * Protocol 4-10 and below use 12 byte VACL without TTL.
		 */

		bzero(&vacl, sizeof(VACL));
		bcopy(data, &vacl, 12);
	}
	else
	{	
		/*
		 * Wrong packet length.
		 */

		vcp_msg(ptr, "* Argument supplied for ACL_ADD has invalid size (%d instead of %d)",
			len, ptr->protocol < 0x00040011 ? 12 : sizeof(VACL));

		return;
	}

	/*
	 * Convert byte order.
	 */

	vacl.rule	= ntohs(vacl.rule);
	vacl.action	= ntohs(vacl.action);
	vacl.ttl	= ntohl(vacl.ttl);

	if(vacl.rule < 65535)
	{
		if(vacl.action == ACL_ACT_ACCEPT
		|| vacl.action == ACL_ACT_REJECT)
		{
			if(vacl.ttl <= DYNACL_MAX_TTL)
			{
				ACLRULE	new;

				bzero(&new, sizeof(new));

				new.rule = vacl.rule;
				new.action = vacl.action == ACL_ACT_ACCEPT ?
						ACL_ACCEPT : ACL_REJECT;

				bcopy(&vacl.address, &new.address, sizeof(new.address));
				bcopy(&vacl.netmask, &new.netmask, sizeof(new.netmask));

				if(vacl.ttl)
				{
					time(&new.expires);
					new.expires += vacl.ttl;
				}

				if(!acldb_add(&new))
				{
					char buf[256];

					acldb_print(&new, buf, sizeof(buf));

					vcp_msg(ptr, "* %s", buf);
				}
				else
				{
					vcp_msg(ptr, "* Failed to add ACL rule (%s%s%s)",
						S_FG_RED, strerror(errno), S_OFF);
				}
			}
			else
			{
				vcp_msg(ptr, "* Invalid TTL specified; maximum=%d", DYNACL_MAX_TTL);
			}
		}
		else
		{
			vcp_msg(ptr, "* Supplied action is invalid");
		}
	}
	else
	{
		vcp_msg(ptr, "* Supplied rule number out of range - use a value between 0 and 65534");
	}
}
