/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: cmd_addnick.c,v 1.9 2009/08/14 11:54:49 mickey Exp $ */

#include "vserver.h"

/*
 * Add a new authority record to the authority database
 */

void cmd_addnick(CLDATA *ptr, void *data, size_t len)
{
	if(len)
	{
		char *nick = (char *)data;
		char *passwd = strchr(nick, ' ');

		if(passwd)
			*passwd++ = 0;

		if(passwd && *passwd)
		{
			size_t plen = strlen(passwd);

			if(plen >= MINPASSWDSIZE && plen <= VPPASSWDSIZE)
			{
				if(*nick)
				{
					if(nick_ok(nick))
					{
						if(!userdb_add(nick, passwd))
						{
							if(userdb_save() > 0)
							{
								vcp_msg(ptr, "* Added %s%s%s to the user database",
									S_FG_YEL, nick, S_OFF);
							}
							else
							{
								vcp_msg(ptr, "* Failed to save user database (%s%s%s)",
									S_FG_RED, strerror(errno), S_OFF);

								userdb_delete(nick);
							}
						}
						else if(errno == EEXIST)
						{
							vcp_msg(ptr, "* Nickname '%s%s%s' is already registered",
								S_FG_RED, nick, S_OFF);
						}
						else if(errno == EBUSY)
						{
							vcp_msg(ptr, "* Nickname %s%s%s is currently in use",
								S_FG_YEL, nick, S_OFF);

						}
						else
						{
							vcp_msg(ptr, "* User database failure (%s%s%s)"
								S_FG_RED, strerror(errno), S_OFF);
						}
					}
					else
					{
						vcp_msg(ptr, "* Nickname '%s%s%s' is invalid",
							S_FG_RED, nick, S_OFF);
					}
				}
				else
				{
					vcp_msg(ptr, "* No nickname supplied");
				}
			}
			else
			{
				vcp_msg(ptr, "* Please choose a password %d "
					"- %d characters in length",
					MINPASSWDSIZE, VPPASSWDSIZE);
			}
		}
		else
		{
			vcp_msg(ptr, "* You must supply a password");
		}
	}
	else
	{
		vcp_msg(ptr, "* Missing argument");
	}
}
