/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: cmd_asstopic.c,v 1.9 2009/08/14 11:54:49 mickey Exp $ */

#include "vserver.h"

/*
 * Assign the specified topic to the current conference channel of
 * the calling client.
 */

void cmd_asstopic(CLDATA *ptr, void *data, size_t len)
{
	char *topic = (char *)data;

	/*
	 * Remove control characters, leading and trailing whitespace
	 * from the topic, and truncate it to the maximum topic size.
	 */

	trimstr(topic, VPTOPICSIZE);

	/*
	 * If anything remains after that, assign the topic, else
	 * request the client to set a topic again.
	 */

	if(*topic)
	{
		if(!topicdb_set(ptr->channel, topic))
		{
			vcp_msg(ptr, "* Topic for channel %s%d%s is '%s%s%s'",
				S_FG_BLU, ptr->channel, S_OFF,
				S_BOLD, topic, S_OFF);
		}
		else if(errno == EPERM)
		{
			vcp_msg(ptr, "* This channel has a fixed topic that may not "
				"be updated");
		}
		else
		{
			vcp_msg(ptr, "* Failed to assign topic to channel %s%d%s (%s)",
				S_FG_BLU, ptr->channel, S_OFF,
				S_FG_RED, strerror(errno), S_OFF);
		}
	}
	else
	{
		vcp_snd(ptr, SUBMIT_TOPIC, NULL, 0);
	}
}
