/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: cmd_chanlist.c,v 1.8 2009/08/14 11:54:49 mickey Exp $ */

#include "vserver.h"

/*
 * List all currently active conference channels and their topics
 */

void cmd_chanlist(CLDATA *ptr, void *data, size_t len)
{
	int topics = topicdb_records();

	if(topics > 0)
	{
		register VTOPIC	*vt = topicdb_addr(0);

		if(vt)
		{
			register int i;
			TABLE cltab;

			table_init(&cltab, ptr);

			table_printf(&cltab, "\n*%s%s     CHANNEL USER TOPIC                                           IDLE       %s",
				S_BG_BLU, S_FG_WHT, S_OFF);

			for(i = 0; i < topics; i++, vt++)
			{	
				if(vt->channel >= 0)
				{
					table_printf(&cltab, "\n* %11d %4d %s%s%*s%s",
						vt->channel,
						chatters(vt->channel),
						vt->topic, S_OFF,
						48 - dspwidth(vt->topic), " ",
						elapsed_now(vt->stamp));
				}
				else
				{
					table_printf(&cltab, "\n*      ?????? %4d %s%s%*s%s",
						chatters(vt->channel),
						vt->topic, S_OFF,
						48-dspwidth(vt->topic), " ",
						elapsed_now(vt->stamp));
				}
			}

			table_flush(&cltab);
		}
		else
		{
			vcp_msg(ptr, "* Topic database failure (%s%s%s)",
				S_FG_RED, strerror(errno), S_OFF);
		}
	}
	else if(!topics)
	{
		vcp_msg(ptr, "* No topic is assigned to any channel");
	}
	else
	{
		vcp_msg(ptr, "* Topic database failure (%s%s%s)",
			S_FG_RED, strerror(errno), S_OFF);
	}
}
