/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: cmd_chchan.c,v 1.8 2009/08/14 11:54:49 mickey Exp $ */

#include "vserver.h"

/*
 * Change the current conference channel
 */

void cmd_chchan(CLDATA *ptr, void *data, size_t len)
{
	if(len == sizeof(int32_t))
	{
		int32_t	newchan = ntohl(*(int32_t *)data);

		if(newchan != ptr->channel)
		{
			uint32_t records = cldb_records();
			register CLDATA *cp;
			VTOPIC *vt;

			/*
			 * Inform other users about the client leaving
			 * the old and joining the new channel.
			 */

			if(records && (cp = cldb_addr(0)))
			{
				register int i;

				for(i = 0; i < records; i++, cp++)
				{
					if(cp == ptr)
					{
						continue;
					}
					else if(cp->channel == newchan)
					{
						vcp_msg(cp, "* %s%s%s joined this channel",
							S_FG_YEL, ptr->nick, S_OFF);
					}
					else if(cp->channel == ptr->channel)
					{
						vcp_msg(cp, "* %s%s%s left this channel",
							S_FG_YEL, ptr->nick, S_OFF);
					}
				}
			}

			/*
			 * Remove topic from old channel if
			 * no users remain on it.
			 */

			topicdb_expire(ptr->channel);

			/*
			 * Change the client's channel and acknowledge.
			 */

			ptr->channel = newchan;

			vcp_snd(ptr, CHAN_ACK, data, len);

			vcp_msg(ptr, "* You are now on channel %s%d%s",
				S_FG_BLU, ptr->channel, S_OFF);

			/*
			 * Tell the client the topic of the new channel
			 * or request a topic to be set if there is none.
			 */

			vt = topicdb_findaddr(newchan);

			if(vt && vt->topic[0])
			{
				vcp_msg(ptr, "* Topic on this channel is '%s%s%s'", 
					S_BOLD, vt->topic, S_OFF);
			}
			else if((!vt && errno == ENOENT)
			|| (vt && !vt->topic[0]))
			{
				topicdb_set(ptr->channel, "<none>");
				vcp_snd(ptr, SUBMIT_TOPIC, NULL, 0);
			}
			else
			{
				vcp_msg(ptr, "* Topic database failure (%s%s%s)",
					S_FG_RED, strerror(errno), S_OFF);
			}
		}
		else
		{
			vcp_msg(ptr, "* You are already on channel %s%d%s",
				S_FG_BLU, newchan, S_OFF);
		}
	}
	else
	{
		vcp_msg(ptr, "* Invalid argument");
	}
}
