/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: cmd_chtopic.c,v 1.9 2009/08/14 11:54:50 mickey Exp $ */

#include "vserver.h"

/*
 * Change the topic of the current conference channel to the specified string.
 */

void cmd_chtopic(CLDATA *ptr, void *data, size_t len)
{
	char *topic = (char *)data;

	/*
	 * Remove control characters, leading and trailing whitespace from
	 * the supplied topic and truncate it to the maximum topic size.
	 */

	trimstr(topic, VPTOPICSIZE);

	/*
	 * If anything remains after that, set the new topic.
	 */

	if(*topic)
	{
		if(!topicdb_set(ptr->channel, topic))
		{
			vcp_msgchan(ptr->channel, (CLDATA *)0,
				"* Topic changed to '%s%s%s' by %s%s%s",
				S_BOLD, topic, S_OFF,
				S_FG_YEL, ptr->nick, S_OFF);

			vlog(VLOG_INFO, "Topic change CHANNEL: %d NEW-TOPIC: '%s' SET-BY: %s",
				ptr->channel, topic, ptr->nick);
		}
		else if(errno == EPERM)
		{
			vcp_msg(ptr, "* This channel has a fixed topic which must not be changed");
		}
		else
		{
			vcp_msg(ptr, "* Topic database failure (%s%s%s)",
				S_FG_RED, strerror(errno), S_OFF);
		}
	}
	else
	{
		vcp_msg(ptr, "* The supplied topic is invalid");
	}
}
