/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: cmd_clopt.c,v 1.9 2009/08/14 11:54:50 mickey Exp $ */

#include "vserver.h"

/*
 * Get/Set client options.
 */

void cmd_clopt(CLDATA *ptr, void *data, size_t len)
{
	TABLE		tab;
	GSOPT		*opt;
	uint32_t	givemask;
	uint32_t	takemask;
	uint32_t	md;

	/*
	 * Check for correct argument length.
	 */

	if(len != sizeof(GSOPT))
	{
		vcp_msg(ptr, "* Invalid packet length encountered");
		return;
	}

	/*
	 * Retrieve arguments
	 */

	opt = (GSOPT *)data;

	givemask = ntohl(opt->givemask);
	takemask = ntohl(opt->takemask);
	md = 0;

	/*
	 * Check for give/take options that will compensate each other
	 */

	md = (givemask ^ takemask);
	givemask &= md;
	takemask &= md;

	/*
	 * Modify client options (unknown options are silently ignored)
	 */

	if(givemask & CL_OPT_WAKEUP)
		ptr->cl_opts |= CL_OPT_WAKEUP;
	if(givemask & CL_OPT_SGNMSG)
		ptr->cl_opts |= CL_OPT_SGNMSG;
	if(givemask & CL_OPT_PRVECHO)
		ptr->cl_opts |= CL_OPT_PRVECHO;

	if(takemask & CL_OPT_WAKEUP)
		ptr->cl_opts &= ~CL_OPT_WAKEUP;
	if(takemask & CL_OPT_SGNMSG)
		ptr->cl_opts &= ~CL_OPT_SGNMSG;
	if(takemask & CL_OPT_PRVECHO)
		ptr->cl_opts &= ~CL_OPT_PRVECHO;

	/*
	 * Print the current option set up
	 */

	table_init(&tab, ptr);

	table_printf(&tab, "* Client options: ");

	display_clopt(&tab, ptr->cl_opts);

	table_flush(&tab);
}

void display_clopt(TABLE *cltab, uint32_t opts)
{
	if(opts & CL_OPT_WAKEUP)
		table_printf(cltab, "%s+%sWAKEUP", S_FG_GRN, S_OFF);
	else
		table_printf(cltab, "%s-%sWAKEUP", S_FG_RED, S_OFF);

	if(opts & CL_OPT_SGNMSG)
		table_printf(cltab, " %s+%sSGNMSG", S_FG_GRN, S_OFF);
	else
		table_printf(cltab, " %s-%sSGNMSG", S_FG_RED, S_OFF);

	if(opts & CL_OPT_PRVECHO)
		table_printf(cltab, " %s+%sPRVECHO", S_FG_GRN, S_OFF);
	else
		table_printf(cltab, " %s-%sPRVECHO", S_FG_RED, S_OFF);

}
