/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: cmd_delnick.c,v 1.11 2009/08/14 11:54:50 mickey Exp $ */

#include "vserver.h"

/*
 * Delete a user from the user database.
 */

void cmd_delnick(CLDATA *ptr, void *data, size_t len)
{
	register int	i;
	TABLE		cltab;
	char		*xargv[VCMD_MAXARGS];
	int		xargc;
	char		modified;

	/*
	 * Initialize output table.
	 */

	table_init(&cltab, ptr);

	/*
	 * Parse nicknames into argument vector.
	 */

	if(!(xargc = nicksplit(&cltab, (char *)data, xargv, VCMD_MAXARGS, 1)))
		return;

	/*
	 * Process nickname list.
	 */

	for(i = 0, modified = 0; i < xargc; i++)
	{
		if(!userdb_delete(xargv[i]))
		{
			table_xprintf(&cltab, "\n* Nickname %s%s%s deleted "
				"from user database", S_FG_YEL, xargv[i],
				S_OFF);

			modified++;
		}
		else if(errno == EPERM)
		{
			table_printf(&cltab, "\n* The '%s%s%s' user may not "
				"be deleted", S_FG_RED, xargv[i], S_OFF);
		}
		else if(errno == ENOENT)
		{
			table_xprintf(&cltab, "\n* Nickname '%s%s%s' is not "
				"registered", S_FG_RED, xargv[i], S_OFF);
		}
		else if(errno == EBUSY)
		{
			table_xprintf(&cltab, "\n* Nickname '%s%s%s' is "
				"currently in use", S_FG_RED, xargv[i], S_OFF);
		}
		else
		{
			table_xprintf(&cltab, "\n* Failed to delete nickname "
				"'%s%s%s' - User database failure (%s%s%s)",
				S_FG_RED, xargv[i], S_OFF,
				S_FG_RED, strerror(errno), S_OFF);
		}
	}

	/*
	 * Sync user database file if modifications where made.
	 */

	if(modified)
	{
		if(userdb_save() < 0)
		{
			table_xprintf(&cltab, "\n* Failure updating user "
				"database file (%s%s%s) - inform system "
				"operator", S_FG_RED, strerror(errno), S_OFF);
		}
		else
		{
			table_xprintf(&cltab, "\n* Successfully updated user "
				"database file");
		}
	}

	/*
	 * Flush output table.
	 */

	table_flush(&cltab);
}
