/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: cmd_fixchan.c,v 1.10 2009/08/14 11:54:50 mickey Exp $ */

#include "vserver.h"

/*
 * Create/Update/Remove fixed channel topics.
 */

void cmd_fixchan(CLDATA *ptr, void *data, size_t len)
{
	if(len > sizeof(int32_t))
	{
		int32_t	channel = ntohl(*(int32_t *)data);
		char *topic = (char *)(data + sizeof(int32_t));

		/*
		 * Remove control characters, leading and trailing whitespace
		 * from topic and truncate it to the maximum topic size.
		 */

		trimstr(topic, VPTOPICSIZE);

		/*
		 * If anything remains after that, fix the topic.
		 */

		if(*topic)
		{
			if(!topicdb_fix(channel, topic))
			{
				vcp_msgchan(channel, (CLDATA *)0,
					"* Topic changed to '%s%s%s' by %s%s%s",
					S_BOLD, topic, S_OFF,
					S_FG_YEL, ptr->nick, S_OFF);

				vcp_msg(ptr, "* Topic for channel %s%d%s has been fixed",
					S_FG_BLU, channel, S_OFF);
			}
			else
			{
				vcp_msg(ptr, "* Failed to fix topic for channel %s%d%s (%s%s%s)",
					S_FG_BLU, channel, S_OFF,
					S_FG_RED, strerror(errno), S_OFF);
			}
		}
		else
		{
			vcp_msg(ptr, "* The supplied topic is invalid");
		}
	}
	else if(len == sizeof(int32_t))
	{
		int32_t channel = ntohl(*(int32_t *)data);

		if(!topicdb_unfix(channel))
		{
			vcp_msg(ptr, "* Topic for channel %s%d%s has been unfixed",
				S_FG_BLU, channel, S_OFF);
		}
		else if(errno == ENOENT)
		{
			vcp_msg(ptr, "* There is no topic stored for channel %s%d%s",
				S_FG_BLU, channel, S_OFF);
		}
		else if(errno == ENOATTR)
		{
			vcp_msg(ptr, "* Topic for channel %s%d%s is not fixed",
				S_FG_BLU, channel, S_OFF);
		}
		else
		{
			vcp_msg(ptr, "* Failed to unfix topic for channel %s%d%s (%s%s%s)",
				S_FG_BLU, channel, S_OFF,
				S_FG_RED, strerror(errno), S_OFF);
		}
	}
	else
	{
		vcp_msg(ptr, "* Invalid argument");
	}
}
