/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: cmd_invite.c,v 1.11 2009/08/14 11:54:50 mickey Exp $ */

#include "vserver.h"

/*
 * Invite user to join a channel
 */

void cmd_invite(CLDATA *ptr, void *data, size_t len)
{
	TABLE cltab;
	int xargc;
	char *xargv[VCMD_MAXARGS];
	register int i;
	register int n;

	/*
	 * Initialize output table.
	 */

	table_init(&cltab, ptr);

	/*
	 * Parse nicknames into argument vector.
	 */

	if(!(xargc = nicksplit(&cltab, (char *)data, xargv, VCMD_MAXARGS, 1)))
		return;

	/*
	 * Lookup client entries.
	 */

	for(i = n = 0; i < xargc; i++)
	{
		if(!strcmp(xargv[i], ptr->nick))
		{
			table_xprintf(&cltab, "\n* No need to invite yourself");
			continue;
		}

		if(cldb_ison(xargv[i], ptr->channel))
		{
			table_xprintf(&cltab, "\n* No need to invite %s%s%s - %s%s%s is already on your channel",
				S_FG_YEL, xargv[i], S_OFF,
				S_FG_YEL, xargv[i], S_OFF);
			continue;
		}

		if(!vcp_msgnick(xargv[i], "* %s%s%s invites you to join channel %s%d%s",
			S_FG_YEL, ptr->nick, S_OFF,
			S_FG_BLU, ptr->channel, S_OFF))

		{
			table_xprintf(&cltab, "\n* Nickname '%s%s%s' not found",
				S_FG_RED, xargv[i], S_OFF);
		}
		else
		{
			n++;	/* One user invited */
		}
	}

	/*
	 * Report status.
	 */

	if(n >= 1)
	{
		table_xprintf(&cltab, "\n* %s%d%s user%s been invited",
			S_FG_MAG, n, S_OFF, (n == 1 ? " has" : "s have"));
	}

	/*
	 * Flush output table.
	 */

	table_flush(&cltab);
}
