/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: cmd_kill.c,v 1.12 2009/08/14 11:54:50 mickey Exp $ */

#include "vserver.h"

/*
 * Enforce signoff of the specified users.
 */

void cmd_kill(CLDATA *ptr, void *data, size_t len)
{
	TABLE cltab;
	int found;
	int xargc;
	char *xargv[VCMD_MAXARGS];
	register int i;

	/*
	 * Initialize output table.
	 */

	table_init(&cltab, ptr);

	/*
	 * Parse nicknames into argument vector.
	 */

	if(!(xargc = nicksplit(&cltab, (char *)data, xargv, VCMD_MAXARGS, 1)))
		return;

	/*
	 * Step through supplied nickname list.
	 */

	for(i = 0; i < xargc; i++)
	{
		found = cldb_kill(xargv[i], ptr->nick);

		if(found == -1)		/* Forbidden */
		{
			table_xprintf(&cltab, "\n* The '%s%s%s' user may not be killed",
				S_FG_RED, xargv[i], S_OFF);
		}
		else if(!found)		/* Not found */
		{
			table_xprintf(&cltab, "\n* Nickname '%s%s%s' not found",
				S_FG_RED, xargv[i], S_OFF);
		}
		else if(found == 1)	/* One instance found */
		{
			table_xprintf(&cltab, "\n* Marked '%s%s%s' for removal",
				S_FG_YEL, xargv[i], S_OFF);
		}
		else			/* Multiple instances found */
		{
			table_xprintf(&cltab, "\n* Marked %d instances of '%s%s%s' for removal",
				found, S_FG_YEL, xargv[i], S_OFF);
		}
	}

	/*
	 * Flush output table.
	 */

	table_flush(&cltab);
}
