/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: cmd_lastlog.c,v 1.12 2009/08/14 11:54:50 mickey Exp $ */

#include "vserver.h"

/*
 * Show list of users that last logged on.
 */

void cmd_lastlog(CLDATA *ptr, void *data, size_t len)
{
	LLOGDAT		*ldat;
	TABLE		cltab;
	char		dtbuf1[DT_SIZE];
	char		dtbuf2[DT_SIZE];
	char		*xargv[VCMD_MAXARGS];
	int		xargc = 0;
	int		count = 0;
	uint32_t	num		= ntohl(*(uint32_t *)data);
	char		display;
	register int	i;
	register int	q;

	/*
	 * Check if user is registered.
	 */

	if(!(ptr->flags & CFLAG_REGISTERED))
	{
		vcp_msg(ptr, "* Only registered users may use the LASTLOG command");
		return;
	}

	/*
	 * Check if lastlog database present.
	 */

	if(!lastlog)
	{
		vcp_msg(ptr, "* Lastlog is disabled");
		return;
	}

	/*
	 * Apply maximum list entries per request.
	 */

	if(num > LASTLOG_MAX_LIST)
		num = LASTLOG_MAX_LIST;

	/*
	 * Initialize output table.
	 */

	table_init(&cltab, ptr);

	/*
	 * Parse nicknames into argument vector.
	 */

	xargc = nicksplit(&cltab, (char *)(data + sizeof(uint32_t)), xargv, VCMD_MAXARGS, 0);

	/*
	 * Display heading.
	 */

	table_printf(&cltab, "\n*%s%s%s NICKNAME        FROM                     LOGIN@         OUT   TIME          %s",
		S_BG_BLU, S_FG_WHT, S_BOLD, S_OFF);

	/*
	 * Step through lastlog database records.
	 */

	for(i = lastlog->header->used; i >= 0 && count < num; --i)
	{
		/*
		 * Get address of current lastlog database record.
		 */

		if(!(ldat = lastlog_get_record(i)))
		{
			if(lastlog)
			{
				/*
				 * Skip this record.
				 */

				continue;
			}
			else
			{
				/*
				 * Do not try to get further records
				 * if lastlog has been disabled due
				 * to error.
				 */

				break;
			}
		}

		/*
		 * When no nickname list is specified, we want to see the
		 * lastlog record anyway. Otherwise we have to determine if
		 * the current lastlog record matches one of the nicknames
		 * on the list of nicknames we want to see.
		 */

		display = 0;

		if(!(display = (!xargc) ? 1 : 0))
		{
			for(q = 0; q < xargc; q++)
			{
				if(!strcmp(ldat->nickname, xargv[q]))
				{
					display = 1;
					break;
				}
			}
		}

		/*
		 * Display the current lastlog record if desired.
		 */

		if(display)
		{
			strftime(dtbuf1, sizeof(dtbuf1), "%b %d %H:%M",
				localtime(&ldat->signon_stamp));
			dtbuf1[sizeof(dtbuf1)-1] = 0;

			strftime(dtbuf2, sizeof(dtbuf2), "%H:%M",
				localtime(&ldat->signoff_stamp));
			dtbuf2[sizeof(dtbuf2)-1] = 0;

			table_xprintf(&cltab, "\n* %-15s %-24s %s - %s %s",
				ldat->nickname,
				strlen(ldat->hostname) > 24 ?
				iptostr(ldat->ipaddr) : ldat->hostname,
				dtbuf1, dtbuf2,
				elapsed(ldat->signon_stamp,
				ldat->signoff_stamp));

			++count;
		}
	}

	/*
	 * Determine the login time of the first record in the database
	 */

	if((ldat = lastlog_get_record(0)))
	{
		strftime(dtbuf1, sizeof(dtbuf1), DT_FMT,
			localtime(&ldat->signon_stamp));
		dtbuf1[sizeof(dtbuf1)-1] = 0;

		table_xprintf(&cltab, "\n* Lastlog begins: %s%s%s",
			S_BOLD, dtbuf1, S_OFF);
	}

	/*
	 * Flush table.
	 */

	table_flush(&cltab);
}
