/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: cmd_page.c,v 1.12 2009/08/14 11:54:50 mickey Exp $ */

#include "vserver.h"

/*
 * Send a chat request to a user on a remote system via e-mail
 */

void cmd_page(CLDATA *ptr, void *data, size_t len)
{
#ifndef WIN32
	char *dat = (char *)data;
	FILE *pipe;
	FILE *mf;
	char comline[255];
	char line[255];
	char *p;

	/*
	 * Page command is only permitted if OPFL_ENABLE_MAIL is set.
	 */

	if(!(vsopt.flags & VO_ENABLE_MAIL))
	{
#endif
		vcp_msg(ptr, "* Sorry, page command is disabled");
		return;
#ifndef WIN32
	}

	/*
	 * Cut off everything in the supplied argument after the first space.
	 */

	if((p = strchr(dat, ' ')) != NULL)
		*p = '\0';

	/*
	 * Sanity checks.
	 */

	if(strchr(dat, '@') == NULL || strchr(dat, '!') != NULL
	|| strchr(dat, '%') != NULL || strchr(dat, '.') != NULL)
	{
		vcp_msg(ptr, "* Invalid E-Mail address specified.");
		return;
	}

	/*
	 * Construct mail commandline
	 */

	snprintf(comline, sizeof(comline), "mail -s \"VChat invitation.\" %s "
		"1>&2 2>/dev/null", dat);

	/*
	 * Execute mail request
	 */

	if((pipe = popen(comline, "w")))
	{
		fprintf(pipe, "                             VChat invitation\n");
		fprintf(pipe, "                             ================\n\n");

		if(ptr->type == CLIENT_GATEWAY)
		{
			fprintf(pipe, " %s from %s [%s] would like to chat to you on\n",
				ptr->nick, ptr->host, inet_ntoa(ptr->ip_addr));
			fprintf(pipe, " the conference server at %s [%s], channel %d.\n\n",
				ptr->if_name, inet_ntoa(ptr->if_addr), ptr->channel);
		}
		else
		{
			fprintf(pipe, " %s@%s [%s] would like to chat to you on\n",
				ptr->user, ptr->host, inet_ntoa(ptr->ip_addr));
			fprintf(pipe, " the conference server at %s [%s], channel %d.\n\n",
				ptr->if_name, inet_ntoa(ptr->if_addr), ptr->channel);
		}

		/*
		 * Copy in mail invitation text file if present.
		 */

		if((mf = fopen(fn_mail, "r")) != NULL)
		{
			while(fgets(line, 255, mf) != NULL)
			{
				while((p = strrchr(line, '\n')) != NULL)
					*p = '\0';

				while((p = strrchr(line, '\r')) != NULL)
					*p = '\0';

				fprintf(pipe, "%s\n", line);
			}

			fclose(mf);
		}

		if(pclose(pipe) && errno != EINTR)
		{
			vcp_msg(ptr, "* Mail failed.");
			vlog(VLOG_ERR, "Mail command returned status %d (%s) USER: %s NICK: %s TO: %s",
				errno, strerror(errno), ptr->user, ptr->nick, dat);
		}
		else
		{
			vcp_msg(ptr, "* Page sent.");
			vlog(VLOG_INFO, "Page request sent USER: %s NICK: %s TO: %s",
				ptr->user, ptr->nick, dat);
		}
	}
	else
	{
		vcp_msg(ptr, "* Sorry, mail is out of order.");
		vlog(VLOG_ERR, "Attempt to open pipe to mail command returned status %d (%s) USER: %s NICK: %s TO: %s",
			errno, strerror(errno), ptr->user, ptr->nick, dat);
	}
#endif /* !WIN32 */
}
