/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: cmd_setpasswd.c,v 1.10 2009/08/14 11:54:51 mickey Exp $ */

#include "vserver.h"

/*
 * Set the password of the specified user.
 */

void cmd_setpasswd(CLDATA *ptr, void *data, size_t len)
{
	char		*nick = (char *)data;
	char		*pass;

	/*
	 * Check if nickname and password are supplied.
	 */

	if((pass = strchr(nick, ' ')))
	{
		*pass++ = 0;

		/*
		 * May not change own password this way - use CHPASSWD instead.
		 */

		if(strcmp(nick, ptr->nick))
		{
			/*
			 * Check the supplied password.
			 */

			if(*pass)
			{
				size_t plen = strlen(pass);

				if(plen >= MINPASSWDSIZE
				&& plen <= VPPASSWDSIZE)
				{
					USERDATA *udat;

					/*
					 * Get user database record for
					 * the supplied nickname.
					 */

					if((udat = userdb_findaddr(nick)))
					{
						char oldpass[VPPASSWDSIZE+1];

						/*
						 * Set the new password and sync database file
						 */

						sncopy(udat->passwd, oldpass, sizeof(oldpass));
						sncopy(pass, udat->passwd, sizeof(udat->passwd));

						if(userdb_save() > 0)
						{
							vcp_msg(ptr, "* Password for %s%s%s "
								"successfully changed", S_FG_YEL,
								nick, S_OFF);
						}
						else
						{
							/*
							 * Failed to save user database to disk.
							 * Restore the old password and try
							 * saving the database again.
							 */

							vcp_msg(ptr, "* Failed to update user database "
								"file (%s%s%s)", S_FG_RED, strerror(errno),
								S_OFF);

							sncopy(oldpass, udat->passwd, sizeof(udat->passwd));
							userdb_save();

							vcp_msg(ptr, "* Password unchanged");
						}

					}
					else if(errno == ENOENT)
					{
						/*
						 * Specified nickname not
						 * found in user database.
						 */

						vcp_msg(ptr, "* Nickname "
							"'%s' is currently not"
							" registered", nick);
					}
					else
					{
						vcp_msg(ptr, "* User database"
							"failure (%s%s%s)",
							S_FG_RED,
							strerror(errno),
							S_OFF);
					}
				}
				else
				{
					vcp_msg(ptr, "* Please choose a "
						"password %d - %d characters "
						"in length", MINPASSWDSIZE,
						VPPASSWDSIZE);
				}
			}
			else
			{
				vcp_msg(ptr, "* Empty passwords are not "
					"allowed");
			}
		}
		else
		{
			vcp_msg(ptr, "* You may not change your own password"
				" using SETPASSWD");
		}
	}
	else
	{
		vcp_msg(ptr, "* Invalid argument");
	}
}
