/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: cmd_svdnsopt.c,v 1.9 2009/08/14 11:54:51 mickey Exp $ */

#include "vserver.h"

/*
 * Get/Set server DNS check options.
 */

void cmd_svdnsopt(CLDATA *ptr, void *data, size_t len)
{
	uint8_t lvl;

	if(!len || (len == 1 && (lvl = *(uint8_t *)data) <= SV_DNS_CHK_STRICT))
	{
		/*
		 * Set DNS check level if argument is present.
		 */

		if(len)
		{
			/*
			 * If the new DNS check level is more secure than
			 * the current setting, signoff all clients that
			 * do not fullfill the new requirement.
			 */

			if(lvl > vsopt.dnschklvl)
			{
				cldb_dnschklvl(lvl);
			}

			/*
			 * Set new DNS checking level.
			 */

			vsopt.dnschklvl = lvl;

			save_options();
		}

		/*
		 * Display current setting.
		 */

		vcp_msg(ptr, "* DNS checking: %s%s%s",
			S_FG_GRN, dnslvlname(vsopt.dnschklvl), S_OFF);
	}
	else
	{
		vcp_msg(ptr, "* Invalid argument");
	}
}
