/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: cmd_svopt.c,v 1.13 2009/08/14 11:54:51 mickey Exp $ */

#include "vserver.h"

/*
 * Get/Set server options.
 */

void cmd_svopt(CLDATA *ptr, void *data, size_t len)
{
	if(len == sizeof(GSOPT))
	{
		TABLE		tab;
		GSOPT		*opt;
		uint32_t	givemask;
		uint32_t	takemask;
		uint32_t	md;

		/*
		 * Retrieve arguments.
		 */

		opt = (GSOPT *)data;
		givemask = ntohl(opt->givemask);
		takemask = ntohl(opt->takemask);
		md = 0;

		/*
		 * Check for give/take options that will compensate each other
		 */

		md = (givemask ^ takemask);
		givemask &= md;
		takemask &= md;

		/*
		 * Modify server options (unknown options are silently ignored)
		 */

		if(givemask & SV_OPT_SHOW_IP)
			vsopt.flags &= ~VO_ANONYMOUS;

		if(givemask & SV_OPT_MAIL_INV)
			vsopt.flags |= VO_ENABLE_MAIL;

		if(givemask & SV_OPT_REG_ONLY)
		{
			uint32_t records = cldb_records();
			register CLDATA	*cp;

			vsopt.flags |= VO_REGONLY;

			/*
			 * Signoff all clients that are not registered.
			 */

			if(records && (cp = cldb_addr(0)))
			{
				register int	i;

				for(i = 0; i < records; i++, cp++)
				{
					if(cp->flags & CFLAG_REGISTERED)
						continue;

					cp->flags |= CFLAG_SIGNOFF;
					snprintf(cp->offmsg, sizeof(cp->offmsg),
						"not registered");
				}
			}
		}

		if(givemask & SV_OPT_FLOOD_GUARD)
		{
			vsopt.flags |= VO_FLOODGUARD;
			vlog(VLOG_NOTICE, "FloodGuard enabled, treshold=%s",
				mt_print(&vsopt.t_guard));
		}

		if(takemask & SV_OPT_SHOW_IP)
			vsopt.flags |= VO_ANONYMOUS;

		if(takemask & SV_OPT_MAIL_INV)
			vsopt.flags &= ~VO_ENABLE_MAIL;

		if(takemask & SV_OPT_REG_ONLY)
			vsopt.flags &= ~VO_REGONLY;

		if(takemask & SV_OPT_FLOOD_GUARD)
		{
			vsopt.flags &= ~VO_FLOODGUARD;
			vlog(VLOG_NOTICE, "FloodGuard disabled");
		}

		/*
		 * Save server options.
		 */

		save_options();

		/*
		 * Print the current option set up
		 */

		table_init(&tab, ptr);

		table_printf(&tab, "* Server options: ");

		display_svopt(&tab);

		table_flush(&tab);
	}
	else
	{
		vcp_msg(ptr, "* Invalid argument");
	}

}

void display_svopt(TABLE *cltab)
{
	if(vsopt.flags & VO_ANONYMOUS)
		table_printf(cltab, "%s-%sSHOW_IP", S_FG_RED, S_OFF);
	else
		table_printf(cltab, "%s+%sSHOW_IP", S_FG_GRN, S_OFF);

	if(vsopt.flags & VO_ENABLE_MAIL)
		table_printf(cltab, " %s+%sMAIL_INV", S_FG_GRN, S_OFF);
	else
		table_printf(cltab, " %s-%sMAIL_INV", S_FG_RED, S_OFF);

	if(vsopt.flags & VO_REGONLY)
		table_printf(cltab, " %s+%sREG_ONLY", S_FG_GRN, S_OFF);
	else
		table_printf(cltab, " %s-%sREG_ONLY", S_FG_RED, S_OFF);

	if(vsopt.flags & VO_FLOODGUARD)
		table_printf(cltab, " %s+%sFLOOD_GUARD", S_FG_GRN, S_OFF);
	else
		table_printf(cltab, " %s-%sFLOOD_GUARD", S_FG_RED, S_OFF);
}
