/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: cmd_ulist.c,v 1.9 2009/08/14 11:54:51 mickey Exp $ */

#include "vserver.h"

/*
 * Show list of chatting users
 */

void cmd_ulist(CLDATA *ptr, void *data, size_t len)
{
	struct list	*lt;

	/*
	 * Check supplied argument size.
	 */

	if(len != sizeof(struct list))
	{
		vcp_msg(ptr, "* Invalid argument");
		return;
	}

	/*
	 * Get supplied argument and do byteorder conversion.
	 */

	lt = (struct list *)data;
	lt->cmd = ntohl(lt->cmd);
	lt->chan = ntohl(lt->chan);

	if(lt->cmd == CHAN && lt->chan < 0)
	{
		vcp_msg(ptr, "* No informations on negative channels available");
	}
	else
	{
		uint32_t records = cldb_records();
		register CLDATA	*cp;
		TABLE cltab;
		int n = 0;

		table_init(&cltab, ptr);

		if(records && (cp = cldb_addr(0)))
		{
			register int i;

			for(i = 0; i < records; i++, cp++)
			{
				/*
				 * Skip over entries not matching the criteria.
				 */

				if(!(lt->cmd == ALL_CHAN
				|| (lt->cmd == MY_CHAN && cp->channel == ptr->channel)
				|| (lt->cmd == CHAN && cp->channel == lt->chan)))
				{
					continue;
				}

				/*
				 * Show header before first entry.
				 */

				if(!n)
				{
					table_printf(&cltab, "\n* %s%d%s user%s online\n*\n"
						"*%s%s NICKNAME            CHANNEL IDLE      T %s%s",
						S_FG_RED, records, S_OFF,
						records == 1 ? "" : "s",
						S_FG_WHT, S_BG_BLU,
						(vsopt.flags & VO_ANONYMOUS) ?
						"" : "FROM                                ",
						S_OFF);
				}

				/*
				 * Show the entry.
				 */

				if(cp->channel < 0)
				{
					table_xprintf(&cltab, "\n* %-15s     ??????? %-9s %c %s",
						cp->nick, elapsed_now(cp->stamp),
						(cp->type == CLIENT_FRONTEND) ? 'F' : 'G',
						(vsopt.flags & VO_ANONYMOUS) ?
						"" : cp->host);
				}
				else
				{
					table_xprintf(&cltab, "\n* %-15s %11d %-9s %c %s",
						cp->nick, cp->channel,
						elapsed_now(cp->stamp),
						(cp->type == CLIENT_FRONTEND) ? 'F' : 'G',
						(vsopt.flags & VO_ANONYMOUS) ?
						"" : cp->host);
				}

				n++;
			}
		}

		if(!n)
		{
			table_printf(&cltab, "* No users on channel %s%d%s",
				S_FG_BLU, lt->chan, S_OFF);
		}

		table_flush(&cltab);
	}
}
