/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: cmd_wakeup.c,v 1.13 2009/08/14 11:54:51 mickey Exp $ */

#include "vserver.h"

/*
 * Send wakeup signal to user
 */

void cmd_wakeup(CLDATA *ptr, void *data, size_t len)
{
	uint32_t records;
	register CLDATA	*cp;
	register int y;
	TABLE cltab;
	char *xargv[VCMD_MAXARGS];
	int xargc;
	int ring;

	/*
	 * Initialize output table.
	 */

	table_init(&cltab, ptr);

	/*
	 * Parse nicknames into argument vector.
	 */

	if(!(xargc = nicksplit(&cltab, (char *)data, xargv, VCMD_MAXARGS, 1)))
		return;

	/*
	 * Notify list of users
	 */

	for(y = 0, records = cldb_records(); y < xargc; y++)
	{
		if(!strcmp(xargv[y], ptr->nick))
		{
			table_xprintf(&cltab, "\n* No need to wake yourself up");
			continue;
		}

		ring = 0;

		if(records && (cp = cldb_addr(0)))
		{
			register int i;

			for(i = 0; i < records; i++, cp++)
			{
				if(strcmp(cp->nick, xargv[y]))
					continue;

				if(cp->cl_opts & CL_OPT_WAKEUP)
				{
					/* Send wakeup signal */

					vcp_snd(cp, WAKEUP, &ptr->nick, ptr->nicklen);

					ring = 1;
				}
				else if(!ring)
				{
					ring = -1;
				}
			}
		}

		/*
		 * Report status for current nickname.
		 */

		if(ring > 0)
		{
			table_xprintf(&cltab, "\n* Sent wakeup signal to %s%s%s",
				S_FG_YEL, xargv[y], S_OFF);

			vlog(VLOG_INFO, "Wakeup USER: %s NICK: %s TO: %s",
				ptr->user, ptr->nick, xargv[y]);
		}
		else if(ring < 0)
		{
			table_xprintf(&cltab, "\n* Cannot wake up %s%s%s (%swakeup disabled%s)",
				S_FG_YEL, xargv[y], S_OFF, S_FG_RED, S_OFF);
		}
		else
		{
			table_xprintf(&cltab, "\n* Nickname '%s%s%s' not found",
				S_FG_RED, xargv[y], S_OFF);
		}
	}

	/*
	 * Flush output table.
	 */

	table_flush(&cltab);
}
