/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: global.c,v 1.17 2009/08/14 11:54:51 mickey Exp $ */

#define __GLOBAL_C__ 1	/* Compiling global.c, do not include extern.h */

#include "vserver.h"

/**       **
 * Globals *
 **       **/

/*
 * File names, relative to the base directory.
 */

const char		*fn_userdb	= "user.db";
const char		*fn_acldb	= "acl.db";
const char		*fn_topicdb	= "topic.db";
const char		*fn_lastlog	= "lastlog.db";
const char		*fn_pid		= "vserver.pid";
const char		*fn_mail	= "mail.txt";
const char		*fn_options	= "options";
const char		*fn_welcome	= "welcome.txt";

#ifdef HAVE_UNIX_SOCKETS
const char		*fn_ctlsock	= "vctl";
#endif

/*
 * The program name we have been called with from argv[0].
 */

char			*prog_name;

/*
 * Server socket descriptor and socket address structure.
 */

int			sock;
struct sockaddr_in	sock_in;

/*
 * Log file stream or NULL when not logging to a file.
 */

FILE			*lgf		= NULL;

/*
 * Global configuration data structure.
 */

CFDATA			cfdata;

/*
 * Global server options data structure.
 */

VSOPT			vsopt;

/*
 * Structure for keeping command execution time statistics.
 */

TMSTAT			tmstat;

/*
 * Server status and request counter structure.
 */

VS_STAT			vs;

/*
 * Lastlog database descriptor block.
 */

LASTLOG			*lastlog	= (LASTLOG *)0;


#if HAVE_UNIX_SOCKETS

/*
 * Server control socket and socket address structure.
 */

int			ctlsock		= 0;
struct sockaddr_un	saun;

#endif

#if HAVE_SIGNALS

/*
 * Jump buffer and timeout valid flag used for connection timeouts in VIO.
 */

jmp_buf			vio_jmpbuf;
int			vio_tovalid	= 0;

#endif

/*
 * The permission names.
 */

const char *permnames[] =
{
	"ADD", "DELETE", "CHACC", "CHMOD", "KILL", "BCAST", "ACL", "QUERY",
	"SOPT", "FIX", "STATUS", "EXCLUDE", "", "", "", "",
	"", "", "", "", "", "", "", "",
	"", "", "", "", "", "", "", ""
};

/*
 * The server command vector includes a request counter, the command name,
 * the command handler and the required access permssions for each command.
 */

SVCMD svcmd[] =
{

/*       CNT NAME               HANDLER         PERMISSION        */
/*       -------------------------------------------------------- */
	{ 0, "unknown",		NULL,		0		},
	{ 0, "signoff",		cmd_signoff,	0		},
	{ 0, "chnick",		cmd_chnick,	0		},
	{ 0, "chchan",		cmd_chchan,	0		},
	{ 0, "prvmsg",		cmd_prvmsg,	0		},
	{ 0, "pubmsg",		cmd_pubmsg,	0		},
	{ 0, "invite",		cmd_invite,	0		},
	{ 0, "version",		cmd_version,	0		},
	{ 0, "status",		cmd_status,	AUTH_STATUS	},
	{ 0, "ulist",		cmd_ulist,	0		},
	{ 0, "page",		cmd_page,	0		},
	{ 0, "action",		cmd_action,	0		},
	{ 0, "chanlist",	cmd_chanlist,	0		},
	{ 0, "asstopic",	cmd_asstopic,	0		},
	{ 0, "chtopic",		cmd_chtopic,	0		},
	{ 0, "wakeup",		cmd_wakeup,	0		},
	{ 0, "rtcmp",		cmd_rtcmp,	0		},
	{ 0, "clientinfo",	cmd_clientinfo,	0		},
	{ 0, "signmsg",		cmd_signmsg,	0		},
	{ 0, "chpasswd",	cmd_chpasswd,	0		},
	{ 0, "setpasswd",	cmd_setpasswd,	AUTH_ROOT	},
	{ 0, "addnick",		cmd_addnick,	AUTH_ADD	},
	{ 0, "delnick",		cmd_delnick,	AUTH_DELETE	},
	{ 0, "chmodperm",	cmd_chmodperm,	AUTH_CHMODPERM	},
	{ 0, "chaccperm",	cmd_chaccperm,	AUTH_CHACCPERM	},
	{ 0, "kill",		cmd_kill,	AUTH_KILL	},
	{ 0, "query",		cmd_query,	AUTH_QUERY	},
	{ 0, "broadcast",	cmd_broadcast,	AUTH_BROADCAST	},
	{ 0, "acl_add",		cmd_acl_add,	AUTH_ACL	},
	{ 0, "acl_delete",	cmd_acl_delete,	AUTH_ACL	},
	{ 0, "acl_flush",	cmd_acl_flush,	AUTH_ACL	},
	{ 0, "acl_list",	cmd_acl_list,	AUTH_ACL	},
	{ 0, "lastlog",		cmd_lastlog,	0		},
	{ 0, "clopt",		cmd_clopt,	0		},
	{ 0, "svopt",		cmd_svopt,	AUTH_SOPT	},
	{ 0, "fixchan",		cmd_fixchan,	AUTH_FIXCHAN	},
	{ 0, "ctxtcmd",		NULL,		0		},
	{ 0, "svdnsopt",	cmd_svdnsopt,	AUTH_SOPT	},
	{ 0, "exclude",		cmd_exclude,	AUTH_EXCLUDE	}
};

uint32_t svcmdlen = (sizeof(svcmd) / sizeof(SVCMD));

#ifdef WIN32 /*====================== WIN32 only ===========================*/

/*
 * Service status and service status handle for reporting to the
 * Windows Service Control Manager.
 */

SERVICE_STATUS		ssStatus;
SERVICE_STATUS_HANDLE	sshStatusHandle;

/*
 * Error code and error string variables.
 */

DWORD			dwErr		= 0;
TCHAR			szErr[256];

#endif /*===================================================================*/
