/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: extern.h,v 1.16 2009/08/14 11:54:53 mickey Exp $ */

#ifndef __VSERVER_EXTERN_H__
#define __VSERVER_EXTERN_H__

/**                              **
 * External variables in global.c *
 **                              **/

/*
 * File names.
 */

extern const char		*fn_userdb;
extern const char		*fn_acldb;
extern const char		*fn_topicdb;
extern const char		*fn_lastlog;
extern const char		*fn_pid;
extern const char		*fn_mail;
extern const char		*fn_options;
extern const char		*fn_welcome;

#ifdef HAVE_UNIX_SOCKETS
extern const char		*fn_ctlsock;
#endif

/*
 * The program name we have been called with from argv[0].
 */

extern char			*prog_name;

/*
 * Server socket descriptor and socket address structure.
 */

extern int			sock;
extern struct sockaddr_in	sock_in;

/*
 * Log file stream or NULL when not logging to a file.
 */

extern FILE			*lgf;

/*
 * Global configuration data structure.
 */

extern CFDATA			cfdata;

/*
 * Global server options data structure.
 */

extern VSOPT			vsopt;

/*
 * Structure for keeping command execution time statistics.
 */

extern TMSTAT			tmstat;

/*
 * Server status and request counter structure.
 */

extern VS_STAT			vs;

/*
 * Lastlog database descriptor block.
 */

extern LASTLOG			*lastlog;


  #if HAVE_UNIX_SOCKETS

/*
 * Server control socket and socket address structure.
 */

extern int			ctlsock;
extern struct sockaddr_un	saun;

  #endif

  #if HAVE_SIGNALS
/*
 * Jump buffer and timeout valid flag used for connection timeouts in VIO.
 */

extern jmp_buf			vio_jmpbuf;
extern int			vio_tovalid;

  #endif

/*
 * The permission names.
 */

extern const char		*permnames[32];

/*
 * The server command vector includes a request counter, the command name,
 * the command handler and the required access permssions for each command.
 */

extern SVCMD			svcmd[];
extern uint32_t			svcmdlen;

  #ifdef WIN32 /*===================== WIN32 only ==========================*/

/*
 * Service status and service status handle for reporting to the
 * Windows Service Control Manager.
 */

extern SERVICE_STATUS		ssStatus;
extern SERVICE_STATUS_HANDLE	sshStatusHandle;

/*
 * Error code and error string variables.
 */

extern DWORD			dwErr;
extern TCHAR			szErr[256];

  #endif /*=================================================================*/

#endif /* __VSERVER_EXTERN_H__ */
