/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: options.h,v 1.12 2009/08/14 11:54:53 mickey Exp $ */

/*
 * Various compile-time options.
 * These should rarely need to be changed.
 */

#ifndef __OPTIONS_H__
#define __OPTIONS_H__

#include "vproto/param.h"

/*
 * The minimum size in characters that the vserver will
 * accept for a users password. Passwords shorter than
 * this value will be rejected by the server.
 */

#define	MINPASSWDSIZE		4

/*
 * The size in bytes of some buffers used around somewhere to
 * receive server messages etc.
 */

#define	SVMSGBUF		8192

/*
 * The maximum number of arguments that vserver will support with
 * client commands that accept a list of arguments as input.
 */

#define	VCMD_MAXARGS		24

/*
 * The default client option mask.
 */

#define DEF_CLOPTMASK		(CL_OPT_WAKEUP | CL_OPT_SGNMSG | CL_OPT_PRVECHO)

/*
 * The minimum number of records in the lastlog database.
 */

#define	LASTLOG_MIN		127

/*
 * The maximum number of records in the lastlog database.
 */

#define	LASTLOG_MAX		131071

/*
 * The maximum allowed number of lastlog entries that are displayed
 * per lastlog command.
 */

#define	LASTLOG_MAX_LIST	512

/*
 * The default TTL value for dynamic ACL rules.
 */

#define	DYNACL_DEFAULT_TTL	0x15180		/* 1 day */

/*
 * The maximum allowable TTL for dynamic ACL rules.
 */

#define DYNACL_MAX_TTL		0x76A700	/* 90 days */

/*
 * The first rule number at which dynamic ACLs are being created.
 */

#define	DYNACL_FIRST_RULE	0xD00F

/*
 * The connection timeout in seconds that the server will utilize
 * for client connections. If a read or write operation on the
 * client's socket cannot be performed within the specified time
 * the operation is forcibly aborted and the corresponding client
 * is disconnected.
 */

#define	CONN_TIMEOUT		8

/*
 * Number and size of static cyclic string buffers used by various
 * functions that format some kind of data and return a char pointer
 * to the result. These functions may be called STCB_NUM times
 * without overwriting the result of a previous call.
 */

#define	STCB_NUM		8
#define	STCB_SIZE		256

/*
 * The strftime(3) format string of the standard date format being used
 * and the size of the corresponding buffers used for formatting dates.
 */

#ifdef WIN32
  #define DT_FMT		"%a, %d %b %Y %H:%M:%S %z"
#else
  #define DT_FMT		"%a, %d %b %Y %H:%M:%S %z"
#endif
#define DT_SIZE			64

/*
 * The strftime(3) format used in writing log messages to the log file.
 */

#define DT_LOGFMT		"%Y-%m-%d %H:%M:%S"

/*
 * RMS database reorganization interval.
 * Specifies the number of seconds between checking of all RMS databases
 * for memory segments that may be freed.
 */

#define RMS_REORG_INTERVAL	60

/*
 * The minimum and maximum values for the FloodGuard threshold time (sec).
 */

#define FG_MIN			1000
#define FG_MAX			10000000

/*
 * The service name and description as shown in the SCManager applet.
 */

#ifdef WIN32
  #define SVC_NAME		"VChat"
  #define SVC_DESC		"Provides internet conferencing services"
#endif

#endif /* __OPTIONS_H__ */
