/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: vctlproto.h,v 1.8 2009/08/14 11:54:53 mickey Exp $ */

#ifndef __VCTL_PROTO_H__
#define __VCTL_PROTO_H__

/*
 * VCTL protocol version.
 */

#define VCTL_PROTOCOL	0x0102

/*
 * Server control message types.
 */

#define	VCTL_CMD_ULIST		0x0000
#define	VCTL_CMD_BROADCAST	0x0001
#define	VCTL_CMD_KILL		0x0002
#define	VCTL_CMD_ANONYMOUS	0x0003
#define VCTL_CMD_ENABLEMAIL	0x0004
#define	VCTL_CMD_REGONLY	0x0005
#define	VCTL_CMD_FLOODGUARD	0x0006
#define VCTL_CMD_FGTHRESHOLD	0x0007
#define	VCTL_CMD_SHUTDOWN	0x0008
#define VCTL_CMD_DNSCHKLVL	0x0009
#define VCTL_CMD_SERVERNAME	0x000A
#define	VCTL_CMD_UPTIME		0x000B
#define	VCTL_CMD_ACL_ADD	0x000C
#define	VCTL_CMD_ACL_DELETE	0x000D
#define VCTL_CMD_ACL_FLUSH	0x000E
#define	VCTL_CMD_ACL_LIST	0x000F
#define	VCTL_CMD_USER_CHECK	0x0010
#define VCTL_CMD_USER_ADD	0x0011
#define VCTL_CMD_USER_DELETE	0x0012
#define	VCTL_CMD_CNMSG		0x0013
#define	VCTL_CMD_LASTLOG	0x0014

/*
 * Response message types.
 */

#define	VCTL_MSG_UINFO		0x0000
#define	VCTL_MSG_ONOFF		0x0001
#define	VCTL_MSG_FGTHRESHOLD	0x0002
#define	VCTL_MSG_DNSCHKLVL	0x0003
#define	VCTL_MSG_SERVERNAME	0x0004
#define	VCTL_MSG_UPTIME		0x0005
#define VCTL_MSG_LASTLOG	0x0006
#define	VCTL_MSG_TEXT		0xD00F

/*
 * Message flags.
 */

#define	VCTL_FLG_MSGIT		0x0001	/* Message initiates transaction */
#define	VCTL_FLG_MSGCT		0x0002	/* Message completes transaction */

#define VCTL_FLG_MSGSA		(VCTL_FLG_MSGIT | VCTL_FLG_MSGCT)

/*
 * VCTL control message structure.
 */

typedef struct
{
	uint16_t	type;		/* Message type		*/
	uint16_t	flags;		/* Message flags	*/
	uint16_t	len;		/* Message total length	*/
	uint16_t	version;	/* Protocol version	*/
	char		data[1016];	/* Message data		*/

} VCTL;

#define VCTLSIZE(x)	(x+8)

/*
 * Include VChat protocol parameters.
 */

#include "vproto/param.h"

/*
 * VCTL kill info structure.
 */

typedef struct
{
	int32_t		channel;
	char		nick[VPNICKSIZE+1];
	char		traitor[16];
	uint16_t	target;
#define	VCTL_KILLTARG_USER	0x0001
#define	VCTL_KILLTARG_CHANNEL	0x0002
#define	VCTL_KILLTARG_ALL	0x0003

} VCTL_KILL;


/*
 * VCTL user data structure
 */

typedef struct
{
	char		nick[VPNICKSIZE+1];
	char		host[128];
	struct in_addr	ip;
	int32_t		channel;
	time_t		stamp;

} VCTL_UINFO;

/*
 * VCTL ACL data structure.
 */

typedef struct
{
	uint16_t	rule;
	uint16_t	flags;
#define	VCTL_ACL_ACCEPT	0x0001		/* Rule accepts rather than rejects */
#define	VCTL_ACL_TTL	0x0002		/* Expires field is a relative TTL  */
	struct in_addr	address;
	struct in_addr	netmask;
	time_t		expires;

} VCTL_ACL;

/*
 * VCTL new user data structure.
 */

typedef struct
{
	char		nick[VPNICKSIZE+1];
	char		passwd[VPPASSWDSIZE+1];

} VCTL_NEWUSER;

/*
 * VCTL message header structure for sending messages to users or channels.
 */

typedef struct
{
	char		from[VPNICKSIZE+1];
	char		tty[16];
	char		to[VPNICKSIZE+1];
	int32_t		channel;

} VCTL_CNMSG;

/*
 * VCTL lastlog query and response structures.
 */

typedef struct
{
	uint32_t	records;
	time_t		since;
	time_t		until;
	struct in_addr	address;
	struct in_addr	netmask;
	char		user[VPUSERSIZE+1];
	char		nick[VPNICKSIZE+1];
	char		host[64];

} VCTL_LLQUERY;

typedef struct
{
	time_t		signon;
	time_t		signoff;
	char		user[VPUSERSIZE+1];
	char		nick[VPNICKSIZE+1];
	char		info[40];
	struct in_addr	address;
	char		host[64];

} VCTL_LLRESP;

#endif /* __VCTL_PROTO_H__ */
