/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: types.h,v 1.6 2009/08/14 11:54:53 mickey Exp $ */

#ifndef __VTYPES_H__
#define __VTYPES_H__

#ifndef __VPARAM_H__
  #include <vproto/param.h>
#endif

/**********************************
 * Software identification packet *
 **********************************/

typedef struct
{
	char		ident[VPIDENTSIZE+1];	/* Indetification	*/
	uint32_t	type;			/* Type specification	*/
	uint32_t	proto;			/* Protocol revision	*/
	uint32_t	magic;			/* Magic number		*/

} VIDENT;

/*******************************
 * Server login data structure *
 *******************************/

typedef struct
{
	char		uname[VPUSERSIZE+1];	/* Username		*/
	char		nick[VPNICKSIZE+1];	/* Initial nickname	*/
	struct in_addr	remote_user_ip;		/* addr of remote user	*/
						/* if type == GATEWAY	*/
} VLOGINDAT;

/*************************************
 * Password authentication structure *
 *************************************/

typedef struct
{
	char		passwd[VPPASSWDSIZE+1];	/* Password		*/	

} VPASSWD;

/*******************************************************
 * Access Control List structure used with CMD_ACL_ADD *
 *******************************************************/

typedef struct
{
	struct in_addr	address;
	struct in_addr	netmask;
	uint16_t	rule;
#define	ACL_ACT_ACCEPT	0x0001
#define	ACL_ACT_REJECT	0x0000
	uint16_t	action;
	uint32_t	ttl;

} VACL;

/************************************
 * General purpose messag structure *
 ************************************/

typedef struct
{
	uint32_t	cmd;			/* server message byte	*/
	uint32_t	len;			/* Length of data that	*/
						/* follows this message */
} VMSG;

/********************************
 * Ulist command data structure *
 ********************************/

struct list
{
	uint32_t	cmd;			/* ulist command	*/
	int32_t		chan;			/* channel number	*/

};

/*******************************
 * RTCMP packet data structure *
 *******************************/

typedef struct
{
	struct timeval	stamp;			/* Timestamp		*/
	uint32_t	seq;			/* Sequence number	*/
	uint8_t		dummy[44];		/* Filler		*/

} RTCMP;

/************************
 * GSOPT data structure *
 ************************/

typedef struct
{
	uint32_t	givemask;		/* Give mask		*/
	uint32_t	takemask;		/* Take mask		*/
	uint32_t	control;		/* Reserved		*/

} GSOPT;

#endif /* __VTYPES_H__ */
