/*****************************************************************************
 * This file is part of the VChat conference server distribution.            *
 *                                                                           *
 * Copyright (c) 1993 - 2009 Andreas S. Wetzel                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * The contents of this file are subject to the terms of the GNU General     *
 * Public License (GPL), version 2 only ("the License") as published by the  *
 * Free Software Foundation.                                                 *
 *                                                                           *
 * This Software comes with ABSOLUTELY NO WARRANTY.                          *
 * See the License for more details. You should have received a copy of the  *
 * License along with this distribution; See the file COPYING. If not, write *
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,   *
 * Boston, MA 02110-1301 USA.                                                *
 *****************************************************************************/
 
/* $Id: vcutil.c,v 1.6 2009/09/04 10:20:13 mickey Exp $ */

#if HAVE_CONFIG_H
  #include "config.h"
#endif

#include <stdio.h>
#if HAVE_SYS_TYPES_H
  #include <sys/types.h>
#endif
#if STDC_HEADERS
  #include <stdlib.h>
  #include <stddef.h>
#else
  #if HAVE_STDLIB_H
    #include <stdlib.h>
  #endif
#endif
#if HAVE_STRING_H
  #if !STDC_HEADERS && HAVE_MEMORY_H
    #include <memory.h>
  #endif
  #include <string.h>
#endif
#if HAVE_STRINGS_H
  #include <strings.h>
#endif
#if HAVE_INTTYPES_H
  #include <inttypes.h>
#else
  #if HAVE_STDINT_H
    #include <stdint.h>
  #endif
#endif
#if HAVE_UNISTD_H
  #include <unistd.h>
#endif
#if HAVE_STDARG_H
  #include <stdarg.h>
#else
  #if HAVE_VARARGS_H
    #include <varargs.h>
  #endif
#endif
#if HAVE_CTYPE_H
  #include <ctype.h>
#endif
#if HAVE_LIMITS_H
  #include <limits.h>
#endif
#if TIME_WITH_SYS_TIME
  #include <sys/time.h>
  #include <time.h>
#else
  #if HAVE_SYS_TIME_H
    #include <sys/time.h>
  #else
    #include <time.h>
  #endif
#endif
#if HAVE_ERRNO_H
  #include <errno.h>
#endif
#if HAVE_SYS_SOCKET_H
  #include <sys/socket.h>
#endif
#if HAVE_NETINET_IN_H
  #include <netinet/in.h>
#endif
#if HAVE_ARPA_INET_H
  #include <arpa/inet.h>
#endif
#if HAVE_NETDB_H
  #include <netdb.h>
#endif
#if HAVE_TERMIOS_H
  #include <termios.h>
#else
  #if HAVE_SGTTY_H
    #include <sgtty.h>
  #else
    #if HAVE_SYS_IOCTL_H
      #include <sys/ioctl.h>
    #endif
  #endif
#endif
#ifdef WIN32
  #include <conio.h>
#endif

/*
 * Windows only includes.
 */

#ifdef WIN32
  #include <windows.h>
  #include <process.h>
  #include <winsock.h>
#endif

/*
 * VChat includes.
 */

#define __VCUTIL_C__

#include "compat.h"
#include "vcutil.h"
#include "ascii.h"

/*
 * Globals set by getnextpq().
 */

int		pqindex = 0;
char		*pqname = NULL;
uint32_t	pqargtype = 0;
char		*pqarg = NULL;
uint8_t		pqbool = 0;
uint32_t	pqvalue = 0;
struct in_addr	pqaddress;
struct in_addr	pqnetmask;
time_t		pqdate = 0;

/*
 * Globals used by pqcheck().
 */

uint8_t		pqinteractive = 0;

/*
 * Component path.
 */

char		pqpath[PATH_MAX];
size_t		pqpathlen = 0;
int		pqpathdepth = 0;

/*
 * Global timer statistics structure.
 */

TMSTAT		tmstat;

/*
 * Convenient function to interpret parameters and qualifiers, optionally
 * with arguments of various types from an argument vector as it's input.
 * The list of recognized parameters/qualifiers is supplied via <pql> and
 * the number of items therein via <npql>. The function may be called upon
 * <argc> and <argv> until it returns PQ_EXHAUSTED, which means there are
 * no further arguments left to process in <argv>. The return value is the
 * ID assigned to a parameter/qualifier within the supplied <pq> structure,
 * provided a parameter/qualifier has been recognized. Otherwise only the
 * identification bit that distinguishes a parameter from a qualifier is set.
 * This bit may be checked by using the ISPARAMETER() and ISQUALIFIER() macros.
 * In case of failure additional error bits will be set within the returned
 * value, to indicate the reason of failure. These may be checked by using
 * the PQ_INVALID(), PQ_AMBIGIOUS(), PQ_NOARG(), PQ_REQARG(), PQ_NOPARSE()
 * and PQ_DATEERR() macros. If the PQ_INVALID() macro tests true upon the
 * returned value, then no valid parameter/qualifier name could be recognized
 * from the input at all. If the PQ_AMBIGIOUS() macro tests true, there are
 * more than one parameter/qualifier names that possibly match the input. If
 * the PQ_NOARG() macro tests true, a valid parameter/qualifier was found,
 * but an argument has been supplied to it although it does not expect one.
 * If the PQ_REQARG() macro tests true, a valid parameter/qualifier has
 * been found in the input, but it requires a mandatory argument to be
 * supplied which is not present. If the PQ_NOPARSE() macro tests true,
 * a valid parameter/qualifier name has been encountered, but it required
 * a special type of argument, that could not be successfully parsed from
 * the supplied argument. Finally, if the PQ_DATEERR() macro tests true,
 * a date specification that lies in the past was found were on in the
 * future was expected or vice versa. All parameter/qualifier names are
 * recognized when abbreviated, as long as they are unambigious within
 * the supplied set of parameter/qualifier names. Comparisation of parameter
 * and qualifier names is always done in UPPERCASE, so all parameter/qualifier
 * names within the supplied <pq> should be in UPPERCASE only!
 *
 * INPUT: argc, argv  -> The argument vector to be interpreted.
 *        pql         -> Pointer to a PQLINK type structure, that defines
 *                       the parameters and qualifiers that are to be
 *                       recognized.
 *        result      -> If specified as non-null, a pointer to the PQLINK
 *                       structure of the parameter/qualifiere that has been
 *                       found (if any) is stored to the location it points to.
 *
 * VARIABLES: pqindex     -> Number of arguments parsed from argv[].
 *            pqname      -> Name of current parameter/qualifier.
 *            pqargtype   -> The type of argument that has been expected
 *                           for the current parameter/qualifier.
 *            pqarg       -> The string argument of the current parameter
 *                           or qualifier, or the empty string if no
 *                           argument was present.
 *            pqbool      -> For arguments of type PQARG_BOOL, this variable
 *                           is either set to 1 or to 0 respectively, when
 *                           the input was successfully parsed.
 *            pqvalue     -> For arguments of type PQARG_NUM, this variable
 *                           is set to the numerical value parsed from the
 *                           argument.
 *            pqaddress   -> For arguments of type PQARG_IPADDR or
 *                           PQARG_NETADDR, this variable is set to the
 *                           internet address parsed from the argument.
 *            pqnetmask   -> For arguments of type PQARG_NETADDR, this
 *                           variable is set to the netmask parsed from
 *                           the argument or /32 if not netmask was present.
 *            pqdate      -> For arguments of type PQARG_PDATE or
 *                           PQARG_FDATE, this variable is set to the
 *                           date specification that has been parsed.
 */

uint32_t getnextpq(int argc, char *argv[], PQLINK *pql, PQLINK **result)
{
	register int	i;
	register int	n;
	register PQLINK	*pqlfound = (PQLINK *)0;
	PQDEF		*pqdfound = (PQDEF *)0;
	size_t		npql;
	size_t		namelen;
	uint32_t	type;

	/*
	 * If a result pointer is given, set it to 0,
	 * until we know for sure, that we have a result.
	 */

	if(result)
		*result = (PQLINK *)0;

	/*
	 * Any arguments left to process?
	 */

	for(; pqindex < argc; pqindex++)
	{
		if(*argv[pqindex])
			break;
	}

	if(pqindex >= argc)
		return PQ_EXHAUSTED;

	/*
	 * Qualifier or Parameter?
	 */

	if(*(pqname = argv[pqindex]) == '/')
	{
		/* Qualifier */

		type = QUALIFIER(0);

		++pqname;

		/*
		 * Do we have a name ?
		 */

		if(!*pqname)
		{
			return (type | PQS_INVALID);
		}

	}
	else
	{
		/* Parameter */

		type = PARAMETER(0);
	}

	++pqindex;

	/*
	 * Look for argument.
	 */

	if((pqarg = strchr(pqname, '=')))
		*pqarg++ = 0;
	else
		pqarg = "";

	/*
	 * Convert given parameter/qualifier name to UPPERCASE.
	 */

	upperstr(pqname);

	/*
	 * Find parameter/qualifier names that match the (possibly partial)
	 * name given. Count the number of matches to check for ambiguity and
	 * remember the last match.
	 */

	if(pql)
	{
		npql = pql->nlinks;
		pql = pql->link;
	}
	else
	{
		npql = 0;
	}

	for(i = n = 0, namelen = strlen(pqname); i < npql; i++)
	{
		if(((ISPARAMETER(type) && ISPARAMETER((pql+i)->pqd->id))
		|| (ISQUALIFIER(type) && ISQUALIFIER((pql+i)->pqd->id)))
		&& !strncasecmp(pqname, (pql+i)->pqd->name, namelen))
		{
			pqlfound = (pql+i);
			++n;
		}
	}

	/*
	 * Replace the possibly abbreviated qualifier name with the
	 * complete name if it has been found and set the type of argument.
	 */

	if(n == 1)
	{
		pqdfound = pqlfound->pqd;
		pqname = pqdfound->name;
		pqargtype = pqdfound->argtype;

		if(result)
		{
			*result = pqlfound;
		}
	}

	/*
	 * Check whether the parameter or qualifier is valid
	 * and for argument presence.
	 */

	if(!n)
	{
		return (type | PQS_INVALID);
	}
	else if(n > 1)
	{
		return (type | PQS_AMBIGIOUS);
	}
	else if(PQARG_PRESENCE(pqdfound->argtype) == PQARG_MANDATORY && !*pqarg)
	{
		return (pqdfound->id | PQS_REQARG);
	}
	else if(PQARG_PRESENCE(pqdfound->argtype) == PQARG_NONE && *pqarg)
	{
		return (pqdfound->id | PQS_NOARG);
	}

	/*
	 * Check for correct argument type.
	 */

	if(PQARG_VALTYPE(pqdfound->argtype) == PQARG_BOOL)
	{
		lowerstr(pqarg);

		if(!strcmp(pqarg, "on") || !strcmp(pqarg, "yes")
		|| !strcmp(pqarg, "true") || !strcmp(pqarg, "1"))
		{
			pqbool = 1;
		}
		else if(!strcmp(pqarg, "off") || !strcmp(pqarg, "no")
		|| !strcmp(pqarg, "false") || !strcmp(pqarg, "0"))
		{
			pqbool = 0;
		}
		else
		{
			return (pqdfound->id | PQS_NOPARSE);
		}
	}
	else if(PQARG_VALTYPE(pqdfound->argtype) == PQARG_NUM)
	{
		char *endp;

		pqvalue = strtol(pqarg, &endp, 0);

		if(!(*pqarg && !*endp))
		{
			return (pqdfound->id | PQS_NOPARSE);
		}
	}
	else if(PQARG_VALTYPE(pqdfound->argtype) == PQARG_IPADDR)
	{
#if HAVE_INET_ATON
		if(!inet_aton(pqarg, &pqaddress))
#else
		if((pqaddress.s_addr = inet_addr(pqarg)) == INADDR_NONE)
#endif
		{
			return (pqdfound->id | PQS_NOPARSE);
		}
	}
	else if(PQARG_VALTYPE(pqdfound->argtype) == PQARG_NETADDR)
	{
		char *mask = NULL;

		/*
		 * Parse internet address/netmask in one of the forms:
		 *
		 * a.b.c.d
		 * a.b.c.d/e.f.g.h
		 * a.b.c.d/n
		 */

		upperstr(pqarg);

		if(!strcmp(pqarg, "ANY") || !strcmp(pqarg, "ALL"))
		{
			pqaddress.s_addr = INADDR_ANY;
			pqnetmask.s_addr = INADDR_ANY;
		}
		else if((mask = strchr(pqarg, '/')))
		{
			*mask++ = 0;
#if HAVE_INET_ATON
			if(!inet_aton(pqarg, &pqaddress))
#else
			if((pqaddress.s_addr = inet_addr(pqarg)) == INADDR_NONE)
#endif
			{
				return (pqdfound->id | PQS_NOPARSE);
			}

			if(strchr(mask, '.'))	/* a.b.c.d/e.f.g.h */
			{
#if HAVE_INET_ATON
				if(!inet_aton(mask, &pqnetmask))
#else
				if((pqnetmask.s_addr = inet_addr(mask)) == INADDR_NONE)
#endif
				{
					return (pqdfound->id | PQS_NOPARSE);
				}
			}
			else			/* a.b.c.d/n */
			{
				int mb = atoi(mask);

				if(mb < 0 || mb > 32)
				{
					return (pqdfound->id | PQS_NOPARSE);
				}

				netmask_create(mb, &pqnetmask);
			}
		}
		else
		{
#if HAVE_INET_ATON
			if(!inet_aton(pqarg, &pqaddress))
#else
			if((pqaddress.s_addr = inet_addr(pqarg)) == INADDR_NONE)
#endif
			{
				return (pqdfound->id | PQS_NOPARSE);
			}

			pqnetmask.s_addr = INADDR_NONE;
		}
	}
	else if(PQARG_VALTYPE(pqdfound->argtype) == PQARG_PDATE)
	{
		time_t now;
		time_t *dt = parsedate(pqarg, 0);

		if(!dt)
		{
			return (pqdfound->id | PQS_NOPARSE);
		}

		pqdate = *dt;
		time(&now);

		if(pqdate > now)
		{
			return (pqdfound->id | PQS_DATEERR);
		}
	}
	else if(PQARG_VALTYPE(pqdfound->argtype) == PQARG_FDATE)
	{
		time_t now;
		time_t *dt = parsedate(pqarg, 1);

		if(!dt)
		{
			return (pqdfound->id | PQS_NOPARSE);
		}

		pqdate = *dt;
		time(&now);

		if(pqdate < now)
		{
			return (pqdfound->id | PQS_DATEERR);
		}
	}

	/*
	 * Valid Parameter/Qualifier found.
	 */

	return pqdfound->id;
}

/*
 * Return a string representation for a parameter or qualifier.
 * If <pqlen> is non-NULL, store the length of the formatted string into it.
 */

char *pqfmt(PQDEF *pq, size_t *pqlen)
{
	static char buf[PATH_MAX+1];
	size_t len;

	if(PQARG_PRESENCE(pq->argtype) == PQARG_NONE)
	{
		len = snprintf(buf, PATH_MAX, "%s%s", ISQUALIFIER(pq->id) ?
			"/" : "", pq->name);
	}
	else if(PQARG_PRESENCE(pq->argtype) == PQARG_OPTIONAL)
	{
		len = snprintf(buf, PATH_MAX, "%s%s[=<%s>]",
			ISQUALIFIER(pq->id) ? "/" : "", pq->name, pq->argname);
	}
	else if(PQARG_PRESENCE(pq->argtype) == PQARG_MANDATORY)
	{
		len = snprintf(buf, PATH_MAX, "%s%s=<%s>",
			ISQUALIFIER(pq->id) ? "/" : "", pq->name, pq->argname);
	}
	else
	{
		len =snprintf(buf, PATH_MAX, "<<invalid argtype>>");
	}

	if(pqlen)
		*pqlen = len;

	return buf;
}

/*
 * Add component <name> to the path.
 */

void addpqpath(PQDEF *pqd)
{
	pqpathlen += snprintf(&pqpath[pqpathlen], PATH_MAX - pqpathlen, "%s%s",
		!pqpathlen ? "" : " ", pqfmt(pqd, 0));
	pqpathdepth++;
}

/*
 * Reset the component path.
 */

void resetpqpath(void)
{
	pqpath[0] = 0;
	pqpathlen = 0;
	pqpathdepth = 0;
}

/*
 * Enable/Disable interactive use.
 */

void pqsetinteractive(uint8_t onoff)
{
	pqinteractive = (onoff ? 1 : 0);
}

/*
 * Check for invalid parameters/qualifiers.
 * Returns 0 if successful, -1 otherwise.
 */

int pqchkfail(uint32_t pqid)
{
	int ret = -1;
	char *what = ISQUALIFIER(pqid) ? "Qualifier '/" :
		(pqpathdepth+pqinteractive) < 2 ? "Command '" : "Parameter '";

	if(PQ_INVALID(pqid))
	{
		printf("%s%s%s%s' is not valid.\n", pqpath,
			!pqpathlen ? "" : ": ", what, pqname);
	}
	else if(PQ_AMBIGIOUS(pqid))
	{
		printf("%s%s%s%s' is ambigious.\n", pqpath,
			!pqpathlen ? "" : ": ", what, pqname);
	}
	else if(PQ_REQARG(pqid))
	{
		printf("%s%s%s%s' requires an argument of type %s.\n", pqpath,
			!pqpathlen ? "" : ": ", what, pqname,
			PQARG_FMT_VALTYPE(pqargtype));
	}
	else if(PQ_NOARG(pqid))
	{
		printf("%s%s%s%s' expects no argument.\n", pqpath,
			!pqpathlen ? "" : ": ", what, pqname);
	}
	else if(PQ_NOPARSE(pqid))
	{
		printf("%s%s%s%s' expects an argument of type %s.\n",
			pqpath, !pqpathlen ? "" : ": ", what, pqname,
			PQARG_FMT_VALTYPE(pqargtype));
	}
	else if(PQ_DATEERR(pqid))
	{
		printf("%s%s%s%s' expects a date in the %s.\n",
			pqpath, !pqpathlen ? "" : ": ", what, pqname,
			PQARG_VALTYPE(pqargtype) == PQARG_PDATE ? "past" : "future");
	}
	else
	{
		/* OK */

		ret = 0;
	}

	return ret;
}

/*
 * Check that the argument list in <argc> and <argv> is exhausted.
 */

int pqcheckend(int argc, char *argv[])
{
	uint32_t pqid;

	pqindex = 0;

	while((pqid = getnextpq(argc, argv, 0, 0)) != PQ_EXHAUSTED)
	{
		if(pqchkfail(pqid))
		{
			return -1;
		}
	}

	return 0;
}

int pqstdfunc(int argc, char *argv[], PQLINK *pql)
{
	uint32_t pqid;
	PQLINK *pqlnk = (PQLINK *)0;

	pqindex = 0;

	while((pqid = getnextpq(argc, argv, pql, &pqlnk)) != PQ_EXHAUSTED)
	{
		if(pqchkfail(pqid))
		{
			return -1;
		}
		else if(pqlnk && pqlnk->pqf)
		{
			break;
		}
	}

	argc -= pqindex;
	argv += pqindex;

	if(!pqlnk)
	{
		printf("%s%sA required parameter is missing.\n", pqpath,
			!pqpathlen ? "" : ": ");
		return -1;
	}

	if(!pqlnk->pqf)
	{
		printf("%s%sNo function assigned to %s.\n", pqpath,
			!pqpathlen ? "" : ": ", pqlnk->pqd->name);
		return -1;
	}

	addpqpath(pqlnk->pqd);

	return pqlnk->pqf(argc, argv, pqlnk);
}

int pqhelp(int argc, char *argv[], PQLINK *pql)
{
	register int i;
	register PQLINK *lp;
	PQLINK *lnk = (PQLINK *)0;
	uint32_t pqid;
	char *what;
	int params;

	pqindex = 0;

	while((pqid = getnextpq(argc, argv, pql, &lnk)) != PQ_EXHAUSTED)
	{
		what = ISQUALIFIER(pqid) ? "Qualifier '/"
			: "Command or parameter '";

		if(PQ_INVALID(pqid))
		{
			printf("%s%s%s%s' is unknown.\n", pqpath,
				!pqpathlen ? "" : ": ", what, pqname);
			return -1;
		}
		else if(PQ_AMBIGIOUS(pqid))
		{
			printf("%s%s%s%s' is ambigious.\n", pqpath,
				!pqpathlen ? "" : ": ", what, pqname);
			return -1;
		}
		else if(lnk && lnk->nlinks)
		{
			pql = lnk;

			argc -= pqindex;
			argv += pqindex;

			pqindex = 0;

			addpqpath(lnk->pqd);
		}
	}

	argc -= pqindex;
	argv += pqindex;

	if(!lnk)
		lnk = pql;

	printf("                                    HELP:\n\n");

	if(pqpathlen)
	{
		printf("%s", pqpath);

		for(i = params = 0, lp = pql->link; i < pql->nlinks; i++, lp++)
		{
			if(ISQUALIFIER(lp->pqd->id))
			{
				printf(" [%s]", pqfmt(lp->pqd, 0));
			}
			else
			{
				params++;
			}
		}

		if(params)
		{
			printf(" ...");
		}

		printf("\n\n");
	}

	printf("%s\n\n", lnk->desc);

	for(i = 0, lp = pql->link; i < pql->nlinks; i++, lp++)
	{
		printf("%-20s - %s\n", pqfmt(lp->pqd, 0), lp->desc);

	}

	printf("\n");

	return 0;
}

void printdate(char *text, struct tm *tm)
{
	char dtbuf[64];
	strftime(dtbuf, sizeof(dtbuf), "%A, %d.%m.%Y %H:%M:%S %z", tm);
	dtbuf[sizeof(dtbuf)-1] = 0;

	printf("%s '%s'\n", text, dtbuf);
}

/*
 * Used internally by parsedate() for checking the plausability
 * of values that have been parsed into the struct <tm>.
 * Optionally a month name (either full or abbreviated) may be
 * supplied, that is converted to it's numerical representation
 * and filled into <tm> before checking plausability. If a weekday
 * name is supplied in <day>, then the tm_wday and tm_mday fields
 * of the supplied <tm> should be initialized to the current date
 * by mktime(), in order to have a reference weekday for shifting
 * the day of the month according to the specified weekday name.
 * The return value is either 1 if the supplied values are fine,
 * or 0 otherwise.
 */

int dc(struct tm *tm, char *mon, char *day)
{
	int ret = 0;
	static time_t ref = 1137326400; /* <- Reference date, Sun 15. Jan 2006 12:00:00 UTC */

	/*
	 * If the name of a month is specified, try to match it
	 * and set the numerical value for it within struct tm.
	 */

	if(mon)
	{
		char buf1[32];
		char buf2[32];
		time_t cmp;
		register int i;
		register struct tm *ctm;

		upperstr(mon);

		for(i = 0, cmp = ref; i < 12; i++, cmp += 0x278D00)
		{
			ctm = gmtime(&cmp);
			strftime(buf1, sizeof(buf1), "%B", ctm);
			strftime(buf2, sizeof(buf2), "%b", ctm);

			upperstr(buf1);
			upperstr(buf2);

			if(!strcmp(mon, buf1) || !strcmp(mon, buf2))
			{
				tm->tm_mon = (i+1);
				break;
			}
		}

		if(i == 12)
		{
			/* Not matched */
			return 0;
		}
	}

	/*
	 * If the name of a weekday is specified, try to match it
	 * and correct the day of month within the struct tm to
	 * that day of the week.
	 */

	if(day)
	{
		char buf1[32];
		char buf2[32];
		time_t cmp;
		register int i;
		register struct tm *ctm;

		upperstr(day);

		for(i = 0, cmp = ref; i < 7; i++, cmp += 0x15180)
		{
			ctm = gmtime(&cmp);
			strftime(buf1, sizeof(buf1), "%A", ctm);
			strftime(buf2, sizeof(buf2), "%a", ctm);

			upperstr(buf1);
			upperstr(buf2);

			if(!strcmp(day, buf1) || !strcmp(day, buf2))
			{
				break;
			}
		}

		if(i == 7)
		{
			/* Not matched */
			return 0;
		}
		else
		{
			/*
			 * Correct the day of month according to the
			 * distance between the current weekday and
			 * the weekday we found.
			 */

			tm->tm_mday -= (tm->tm_wday - i);
		}
	}

	/*
	 * Check the supplied values within struct tm
	 * for plausability, correct for two-digit
	 * year numbers and struct tm fields that
	 * are interpreted differently than parsed.
	 */

	if(tm->tm_sec >= 0 && tm->tm_sec < 60)
	{
		if(tm->tm_min >= 0 && tm->tm_min < 60)
		{
			if(tm->tm_hour >= 0 && tm->tm_hour < 24)
			{
				if(tm->tm_mday > 0 && tm->tm_mday <= 31)
				{
					if(tm->tm_mon > 0 && tm->tm_mon <= 12)
					{
						/*
						 * All seems well so far
						 * Now take care of 2 digit
						 * year specifications.
						 */

						if(!tm->tm_year)
						{
							/* 00 -> 2000 */

							tm->tm_year = 100;
						}
						else if(tm->tm_year >= 1 && tm->tm_year <= 68)
						{
							/* 01-68 -> 2001-2068 */

							tm->tm_year += 100;
						}
						else if(tm->tm_year >= 69 && tm->tm_year <= 99)
						{
							/* 69-99 -> 1969-1999 */
						}
						else
						{
							tm->tm_year -= 1900;
						}

						/*
						 * Correct month, which is
						 * interpreted in struct tm
						 * as 0-11 *yuk*
						 */

						tm->tm_mon -= 1;

						/*
						 * OK, all seems valid so far.
						 */

						ret = 1;
					}
				}
			}
		}
	}

	return ret;
}

/*
 * Parse a string representation of a date.
 *
 * The date string supplied in <arg> is parsed according to a number
 * of recognized formats. The parameter <future> gives a hint wheter
 * a date in the future or in the past is expected. If for example
 * the input date string is 'Friday' and today is 'Tuesday', then either
 * last friday would be assumed with <future> being 0, or next friday
 * when <future> was specified as non 0. If a date has been successfully
 * parsed from the input, then a pointer to time_t value is returned.
 * Otherwise a value of NULL is returned.
 */

time_t *parsedate(char *arg, int future)
{
	static time_t ret = 0;
	struct tm tm;
	char buf[24];

	bzero(&tm, sizeof(tm));
	tm.tm_isdst = -1;

	/*
	 * Stage 1: Try to parse a date with maximum accuracy.
	 */

	if(((6 != sscanf(arg, "%2d.%2d.%4d%*[ \t]%2d:%2d:%2d", &tm.tm_mday, &tm.tm_mon,
	&tm.tm_year, &tm.tm_hour, &tm.tm_min, &tm.tm_sec)
	&& 6 != sscanf(arg, "%4d-%2d-%2d%*[ \t]%2d:%2d:%2d", &tm.tm_year, &tm.tm_mon,
	&tm.tm_mday, &tm.tm_hour, &tm.tm_min, &tm.tm_sec)
	&& 6 != sscanf(arg, "%2d/%2d/%4d%*[ \t]%2d:%2d:%2d", &tm.tm_mon, &tm.tm_mday,
	&tm.tm_year, &tm.tm_hour, &tm.tm_min, &tm.tm_sec)) || !dc(&tm, NULL, NULL))
		&&
	((6 != sscanf(arg, "%2d. %23[^0-9 \t\n\r./-:] %4d%*[ \t]%2d:%2d:%2d", &tm.tm_mday, buf,
	&tm.tm_year, &tm.tm_hour, &tm.tm_min, &tm.tm_sec)
	&& 6 != sscanf(arg, "%2d %23[^0-9 \t\n\r./-:] %4d%*[ \t]%2d:%2d:%2d", &tm.tm_mday, buf,
	&tm.tm_year, &tm.tm_hour, &tm.tm_min, &tm.tm_sec)
	&& 6 != sscanf(arg, "%23[^0-9 \t\n\r./-:] %2d%*[ \t]%4d%*[ \t]%2d:%2d:%2d", buf, &tm.tm_mday,
	&tm.tm_year, &tm.tm_hour, &tm.tm_min, &tm.tm_sec)) || !dc(&tm, buf, NULL)))
	{
		/*
		 * Stage 2: Try without seconds.
		 */

		tm.tm_sec = 0;

		if(((5 != sscanf(arg, "%2d.%2d.%4d%*[ \t]%2d:%2d", &tm.tm_mday, &tm.tm_mon,
		&tm.tm_year, &tm.tm_hour, &tm.tm_min)
		&& 5 != sscanf(arg, "%4d-%2d-%2d%*[ \t]%2d:%2d", &tm.tm_year, &tm.tm_mon,
		&tm.tm_mday, &tm.tm_hour, &tm.tm_min)
		&& 5 != sscanf(arg, "%2d/%2d/%4d%*[ \t]%2d:%2d", &tm.tm_mon, &tm.tm_mday,
		&tm.tm_year, &tm.tm_hour, &tm.tm_min)) || !dc(&tm, NULL, NULL))
			&&
		((5 != sscanf(arg, "%2d. %23[^0-9 \t\n\r./-:] %4d%*[ \t]%2d:%2d", &tm.tm_mday, buf,
		&tm.tm_year, &tm.tm_hour, &tm.tm_min)
		&& 5 != sscanf(arg, "%2d %23[^0-9 \t\n\r./-:] %4d%*[ \t]%2d:%2d", &tm.tm_mday, buf,
		&tm.tm_year, &tm.tm_hour, &tm.tm_min)
		&& 5 != sscanf(arg, "%23[^0-9 \t\n\r./-:] %2d%*[ \t]%4d%*[ \t]%2d:%2d", buf, &tm.tm_mday,
		&tm.tm_year, &tm.tm_hour, &tm.tm_min)) || !dc(&tm, buf, NULL)))
		{
			/*
			 * Stage 3: Try just the date without time.
			 */

			tm.tm_hour = tm.tm_min = 0;

			if(((3 != sscanf(arg, "%2d.%2d.%4d", &tm.tm_mday, &tm.tm_mon, &tm.tm_year)
			&& 3 != sscanf(arg, "%4d-%2d-%2d", &tm.tm_year, &tm.tm_mon, &tm.tm_mday)
			&& 3 != sscanf(arg, "%2d/%2d/%4d", &tm.tm_mon, &tm.tm_mday, &tm.tm_year))
			|| !dc(&tm, NULL, NULL))
				&&
			((3 != sscanf(arg, "%2d. %23[^0-9 \t\n\r./-:] %4d", &tm.tm_mday, buf, &tm.tm_year)
			&& 3 != sscanf(arg, "%2d %23[^0-9 \t\n\r./-:] %4d", &tm.tm_mday, buf, &tm.tm_year)
			&& 3 != sscanf(arg, "%23[^0-9 \t\n\r./-:] %2d%*[ \t]%4d", buf, &tm.tm_mday, &tm.tm_year))
			|| !dc(&tm, buf, NULL)))
			{
				/*
				 * Stage 4: Try month and year.
				 */

				tm.tm_mday = 1;

				if((((2 != sscanf(arg, "%2d.%4d", &tm.tm_mon, &tm.tm_year) || strlen(arg) != 7)
				&& 2 != sscanf(arg, "%4d-%2d", &tm.tm_year, &tm.tm_mon)
				&& 2 != sscanf(arg, "%2d/%4d", &tm.tm_mon, &tm.tm_year))
				|| !dc(&tm, NULL, NULL))
					&&
				((2 != sscanf(arg, "%23[^0-9 \t\n\r./-:] %4d", buf, &tm.tm_year)
				&& 2 != sscanf(arg, "%4d %23[^0-9 \t\n\r./-:]", &tm.tm_year, buf))
				|| tm.tm_year < 32 || !dc(&tm, buf, NULL)))
				{
					/*
					 * Stage 5: Try day and month.
					 */

					time_t now;
					struct tm ref;

					time(&now);
					bcopy(localtime(&now), &ref, sizeof(ref));

					tm.tm_year = ref.tm_year + 1900;

					if(((2 != sscanf(arg, "%2d.%2d.", &tm.tm_mday, &tm.tm_mon)
					&& 2 != sscanf(arg, "%2d-%2d", &tm.tm_mon, &tm.tm_mday)
					&& 2 != sscanf(arg, "%2d/%2d", &tm.tm_mon, &tm.tm_mday))
					|| !dc(&tm, NULL, NULL))
						&&
					((2 != sscanf(arg, "%23[^0-9 \t\n\r./-:] %2d", buf, &tm.tm_mday)
					&& 2 != sscanf(arg, "%2d. %23[^0-9 \t\n\r./-:]", &tm.tm_mday, buf)
					&& 2 != sscanf(arg, "%2d %23[^0-9 \t\n\r./-:]", &tm.tm_mday, buf))
					|| !dc(&tm, buf, NULL)))
					{
						/*
						 * Stage 6: Try month names.
						 */

						tm.tm_mday = 1;

						if(1 != sscanf(arg, " %23[^0-9 \t\n\r./-:]", buf) || !dc(&tm, buf, NULL))
						{
							/*
							 * Stage 7: Try weekday names.
							 */

							tm.tm_mday = ref.tm_mday;
							tm.tm_mon = ref.tm_mon + 1;
							tm.tm_wday = ref.tm_wday;
							tm.tm_yday = ref.tm_yday;

							if(1 != sscanf(arg, " %23[^0-9 \t\n\r./-:]", buf) || !dc(&tm, NULL, buf))
							{
								/*
								 * Stage 8: Try four digit year.
								 */

								tm.tm_mday = tm.tm_mon = 1;
								tm.tm_yday = 0;

								if(1 != sscanf(arg, " %4d", &tm.tm_year)
								|| tm.tm_year < 1000 || !dc(&tm, NULL, NULL))
								{
									/*
									 * The user is *definately* too
									 * dumb, to enter a correct date ;-)
									 */

									return (time_t *)0;
								}
							}
							else
							{
								/*
								 * If the date we get lies in the past,
								 * but we expect a date in the future,
								 * then assume that day one week ahead.
								 * Conversely if we get a date in the
								 * future but expect a date in the past,
								 * assume that day one week ago.
								 */

								time_t parsed = mktime(&tm);

								if(parsed < now && future)
									tm.tm_mday += 7;
								else if(parsed > now && !future)
									tm.tm_mday -= 7;
							}
						}
						else
						{
							/*
							 * If the date we get lies in the past, but we expect
							 * a date in the future, then assume the same date one
							 * year ahead. Conversely, if the date we get lies in
							 * the future and we expect a date in the past, then
							 * assume the same date one year ago.
							 */

							time_t parsed = mktime(&tm);

							if(parsed < now && future)
								tm.tm_year++;
							else if(parsed > now && !future)
								tm.tm_year--;
						}
					}
					else
					{
						/*
						 * If the date we get lies in the past, but we expect
						 * a date in the future, then assume the same date one
						 * year ahead. Conversely, if the date we get lies in
						 * the future and we expect a date in the past, then
						 * assume the same date one year ago.
						 */

						time_t parsed = mktime(&tm);

						if(parsed < now && future)
							tm.tm_year++;
						else if(parsed > now && !future)
							tm.tm_year--;
					}
				}
			}
		}
	}

	ret = mktime(&tm);

	printdate("Date parsed:", &tm);
	return &ret;
}

/*
 * Copy the string pointed to by <src> to the buffer pointed to by <dst>
 * whose size is <len> bytes. If <len> is less than 1, nothing is copied
 * and <dst> is not modified. If <len> is greater or equal to 1, then <dst>
 * is always NUL terminated, thus if <src> points to a string that is longer
 * than <len> - 1 characters it will be truncated. The returned value is the
 * number of characters placed into <dst> not including the terminating NUL
 * character.
 */

size_t sncopy(register char *src, register char *dst, size_t len)
{
	register size_t	count;

	/*
	 * No buffer - No action.
	 */

	if(len-- < 1)
		return 0;

	/*
	 * Copy string until either NUL is hit or buffer space exhausted.
	 */

	for(count = 0; count < len && *src != '\0'; count++)
		*dst++ = *src++;

	/*
	 * NUL terminate the destination string.
	 */

	*dst = '\0';

	/*
	 * Return the number of characters copied.
	 */

	return count;
}

/*
 * Return a string representation for the internet address <ip>
 * that is useable multiple times in printf style function calls
 * without clobbering the buffer of each previous call.
 */

char *iptostr(struct in_addr ip)
{
	static char	buf[STCB_NUM][STCB_SIZE];
	static int	index = 0;

	if(++index >= STCB_NUM)
		index = 0;

	snprintf(&buf[index][0], STCB_SIZE, "%s", inet_ntoa(ip));

	return &buf[index][0];
}

/*
 * Return the DNS name for the host specified by <ip>.
 * If the DNS name cannot be found, the ASCII represantation of the
 * internet address contained in <ip> is returned.
 */

char *iptoname(struct in_addr ip)
{
	static char	buf[STCB_NUM][STCB_SIZE];
	static int	index = 0;
	struct hostent	*ht;

	if(++index >= STCB_NUM)
		index = 0;

	ht = gethostbyaddr((char *)&ip, sizeof(ip), AF_INET);

	snprintf(&buf[index][0], STCB_SIZE, "%s",
		ht == NULL ? inet_ntoa(ip) : ht->h_name);
	
	return &buf[index][0];
}

/*
 * Return the DNS name for the host specified by <ip>
 * or NULL if no matching DNS record could be found.
 */

char *dnsname(struct in_addr ip)
{
	static char	buf[STCB_NUM][STCB_SIZE];
	static int	index = 0;
	struct hostent	*ht;

	if((ht = gethostbyaddr((char *)&ip, sizeof(ip), AF_INET)) == NULL)
		return NULL;

	if(++index >= STCB_NUM)
		index = 0;

	snprintf(&buf[index][0], STCB_SIZE, "%s", ht->h_name);

	return &buf[index][0];
}

/*
 * Convert the string pointed to by <str> into lower case and return <str>.
 */

char *lowerstr(char *str)
{
	register char *s;

	for(s = str; *s; s++)
		*s = tolower(*s);

	return str;
}

/*
 * Convert the string pointed to by <str> into upper case and return <str>.
 */

char *upperstr(char *str)
{
	register char *s;

	for(s = str ; *s; s++)
		*s = toupper(*s);

	return str;
}

/*
 * Truncate the string pointed to by <str> to at most <maxlen> characters,
 * trim off leading and trailing whitespace and remove unprintable characters
 * from the string.
 */

char *trimstr(register char *str, size_t maxlen)
{
	char buf[maxlen+1];
	register char *s;
	register size_t cnt;

	/*
	 * Copy at most <maxlen> characters to buffer ignoring
	 * non-printable characters and leading whitespace.
	 */

	for(s = str, cnt = 0; *s && cnt < maxlen; s++)
	{
		if(*s > ' ' || (*s == ' ' && cnt))
			buf[cnt++] = *s;
	}

	buf[cnt] = 0;

	/*
	 * Remove trailing whitespace.
	 */

	for( ; cnt > 0 && buf[cnt-1] == ' '; cnt--)
		buf[cnt-1] = 0;

	/*
	 * Copy back buffer.
	 */

	sncopy(buf, str, maxlen);

	return str;
}

/*
 * Calculate the display width of a color-coded string.
 */

size_t dspwidth(register char *str)
{
	register size_t len;
	register int ign;

	for(len = 0, ign = 0; *str ; str++)
	{
		if(*(uint8_t *)str == CSI)
		{
			ign = 1;
		}
		else
		{
			if(ign)
			{
				ign = 0;
			}
			else
			{
				++len;
			}
		}
	}

	return len;
}

/*
 * Read input from the string pointed to by <str> unescaping any
 * escape sequences beginning with the CSI indicator. At most <len>
 * bytes of output will be written to the buffer pointed to by <buf>.
 * The return value is the number of bytes placed into the buffer <buf>.
 */

size_t unescapestr(uint8_t *str, uint8_t *buf, size_t len)
{
	size_t	cnt	= 0;

	while(str != NULL && *str != '\0' && cnt < len-1)
	{
		if(*str == CSI)
		{
			++str;

			switch(*str)
			{
				case CSI:	*buf++ = CSI;
						++cnt;
						++str;
						break;

				case ' ':	*buf++ = ' ';
						++cnt;
						++str;
						break;

				case '\0':	break;

				default:	++str;
						break;
			}
		}
		else
		{
			*buf++ = *str++;
			++cnt;
		}
	}

	*buf = '\0';

	return cnt;
}

/*
 * Switch terminal modes.
 */

void tty_mode(int mode)
{
#ifndef WIN32
	static int saved = 0;
#if HAVE_TERMIOS_H
	static struct termios termsave;
	struct termios termparam;
#else	/* SGTTY */
	static struct sgttyb ttysave;
	struct sgttyb ttyparam;
#endif

	/*
	 * Save original tty settings on first call.
	 */

	if(!saved)
	{
#if HAVE_TERMIOS_H
		tcgetattr(0, &termsave);
#else
		ioctl(0, TIOCGETP, &ttysave);
#endif
		saved = 1;
	}

	/*
	 * Set up terminal according to supplied mode.
	 */

	if(mode == 0)		/* Restore original tty settings */
	{
		if(saved)
#if HAVE_TERMIOS_H
			tcsetattr(0, TCSANOW, &termsave);
#else
			ioctl(0, TIOCSETP, &ttysave);
#endif
	}
	else if(mode == 1 || mode == 2)	/* CBREAK mode */
	{
#if HAVE_TERMIOS_H
		tcgetattr(0, &termparam);
		termparam.c_lflag &= ~ICANON;

		if(mode == 2)
			termparam.c_lflag &= ~ECHO;
		else
			termparam.c_lflag |= ECHO;

		termparam.c_iflag &= ~ISTRIP;
  #if defined(XTABS)
		termparam.c_iflag &= ~XTABS;
  #elif defined(OXTABS)
		termparam.c_iflag &= ~OXTABS;
  #endif
		termparam.c_cc[VMIN]	= 1;
		termparam.c_cc[VTIME]	= 0;
		tcsetattr(0, TCSANOW, &termparam);
#else /* SGTTY */
		ioctl(0, TIOCGETP, &ttyparam);
		ttyparam.sg_flags |= (CBREAK | PASS8OUT | PASS8);
		ttyparam.sg_flags &= ~XTABS;

		if(mode == 2)
			ttyparam.sg_flags &= ~ECHO;
		else
			ttyparam.sg_flags |= ECHO;

		ioctl(0, TIOCSETP, &ttyparam);
#endif
	}
#endif
}

int promptkey(char *validkeys, char *prompt, ...)
{
	va_list ap;
	int key;
	char *s;

	va_start(ap, prompt);
	vprintf(prompt, ap);
	va_end(ap);

	printf(" [");

	for(s = validkeys; *s; s++)
	{
		printf("%s%c", s == validkeys ? "" : "/", *s);
	}
	printf("]: ");
	fflush(stdout);

#ifdef WIN32
	for(key = -1; strchr(validkeys, key) == NULL; key = tolower(getch()));
#else
	tty_mode(2);
	for(key = -1; strchr(validkeys, key) == NULL; key = tolower(getchar()));
	tty_mode(0);
#endif
	return key;
}

/*
 * Get the number of mask bits in the specified netmask
 */

int netmask_bits(struct in_addr netmask)
{
	uint32_t	mask	= ntohl(netmask.s_addr);
	int		bits	= 0;
	int		flg	= 0;
	int		i;

	for(i = 0; i < (CHAR_BIT * sizeof(uint32_t)); i++)
	{
		if(mask & 0x00000001)
		{
			flg = 1;
			++bits;
		}
		else
		{
			if(flg)
				return -1;
		}

		mask = (mask >> 1);
	}

	return bits;
}

/*
 * Create a netmask with the number of <bits> specified, and store
 * it into the in_addr struct pointed to by <netmask>.
 */

void netmask_create(int bits, struct in_addr *netmask)
{
	netmask->s_addr = htonl(~0 << ((CHAR_BIT * sizeof(uint32_t)) - bits));
}


/*
 * Return a string representation of the time that has elapsed since
 * the time specified by <clock1> and the time specified by <clock2>.
 */

char *elapsed(time_t clock1, time_t clock2)
{
	static char	buf[STCB_NUM][STCB_SIZE];
	static int	index	= 0;

	char		*sgn	= (clock1 > clock2) ? "-" : "";
	time_t		diff	= abs(clock2 - clock1);
	u_int		day	= (diff / 0x15180);
	u_int		hrs	= ((diff % 0x15180) / 0xE10);
	u_int		min	= ((diff % 0xE10) / 0x3C);
	u_int		sec	= (diff % 0x3C);

	if(++index >= STCB_NUM)
		index = 0;

	if(day)
		snprintf(&buf[index][0], STCB_SIZE, "%s%dd+%02d:%02dh", sgn, day, hrs, min);
	else if(hrs)
		snprintf(&buf[index][0], STCB_SIZE, "%s%02d:%02d:%02dh", sgn, hrs, min, sec);
	else if(min)
		snprintf(&buf[index][0], STCB_SIZE, "%s%02d:%02dm", sgn, min, sec);
	else
		snprintf(&buf[index][0], STCB_SIZE, "%s%ds", sgn, sec);

	return &buf[index][0];
}

/*
 * Return a string representation of time that has elapsed since
 * the time specified by <clock> and the current time.
 */

char *elapsed_now(time_t clock)
{
	return elapsed(clock, time(NULL));
}

/*
 * Store a timestamp into the timeval struct pointed to by <tv>.
 */

void mt_stamp(struct timeval *tv)
{
#ifndef WIN32
	struct timezone tz;

	gettimeofday(tv, &tz);
#else
	clock_t	clk	= clock();

	tv->tv_sec = (long)(clk / CLOCKS_PER_SEC);
	tv->tv_usec = (long)((clk % CLOCKS_PER_SEC) * (1000000 / CLOCKS_PER_SEC));
#endif
}

/*
 * Return a pointer to a struct timeval that contains the time span
 * that has elapsed between the time specified by <tv1> and the time
 * specified by <tv2>.
 */

struct timeval *mt_elapsed(struct timeval *tv1, struct timeval *tv2)
{
	static struct timeval elapsed;

	if(tv1->tv_sec == tv2->tv_sec)
	{
		elapsed.tv_sec = 0;
		elapsed.tv_usec = (tv2->tv_usec - tv1->tv_usec);
	}
	else
	{
		if(tv2->tv_usec > tv1->tv_usec)
		{
			elapsed.tv_sec = (tv2->tv_sec - tv1->tv_sec);
			elapsed.tv_usec = (tv2->tv_usec - tv1->tv_usec);
		}
		else
		{
			elapsed.tv_sec = (tv2->tv_sec - tv1->tv_sec - 1);
			elapsed.tv_usec = (1000000 -
				(tv1->tv_usec - tv2->tv_usec));
		}
	}

	return &elapsed;
}

/*
 * Return a pointer to a struct timeval that contains the time span
 * that has elapsed since the time specified by <tv> and the current
 * time.
 */

struct timeval *mt_elapsed_now(struct timeval *tv)
{
	struct timeval now;

	mt_stamp(&now);

	return mt_elapsed(tv, &now);
}

/*
 * Store the time span represented by <tv> for statistical processing.
 */

void mt_store(struct timeval *tv)
{
	/*
	 * Copy value to the next location.
	 */

	bcopy(tv, &tmstat.value[tmstat.next], sizeof(struct timeval));

	/*
	 * Move next index.
	 */

	if(++tmstat.next >= NUMSTATVALS)
		tmstat.next = 0;

	/*
	 * Increment number of used entries until capacity is reached.
	 */

	if(tmstat.used < NUMSTATVALS)
		++tmstat.used;
}

/*
 * Return a string representation of the time span specified by <tv>.
 */

char *mt_print(struct timeval *tv)
{
	static char	buf[STCB_NUM][STCB_SIZE];
	static int	index = 0;

	if(++index >= STCB_NUM)
		index = 0;

	snprintf(&buf[index][0], STCB_SIZE, "%.3f ms",
		((double)tv->tv_usec / 1000) + (tv->tv_sec * 1000));

	return &buf[index][0];
}

/*
 * Report the minimum, average and maximum time spans calculated from the
 * values previously stored by mt_store().
 */

TMREPORT *mt_report(void)
{
	static TMREPORT tmr;
	register int i;

	tmr._min.tv_sec = tmr._min.tv_usec = (tmstat.used) ? 0x7fffffff : 0;
	tmr._avg.tv_sec = tmr._avg.tv_usec = 0;
	tmr._max.tv_sec = tmr._max.tv_usec = 0;

	for(i = 0; i < tmstat.used; i++)
	{
		if(timercmp(&tmstat.value[i], &tmr._min, <))
			bcopy(&tmstat.value[i], &tmr._min, sizeof(tmr._min));

		if(timercmp(&tmstat.value[i], &tmr._max, >))
			bcopy(&tmstat.value[i], &tmr._max, sizeof(tmr._max));

		mt_sum(&tmstat.value[i], &tmr._avg);
	}

	if(tmstat.used)
		mt_avg(&tmr._avg);

	return &tmr;
}

/*
 * Sum the time span specified by <tv> onto <sum>.
 */

void mt_sum(struct timeval *tv, struct timeval *sum)
{
	uint32_t usec;

	if((usec = (sum->tv_usec + tv->tv_usec)) > 1000000)
	{
		sum->tv_sec += (tv->tv_sec + 1);
		sum->tv_usec = (usec - 1000000);
	}
	else
	{
		sum->tv_sec += tv->tv_sec;
		sum->tv_usec = usec;
	}
}

/*
 * Calculate the average value of the time spans previously summed on <sum>.
 */

void mt_avg(struct timeval *sum)
{
	struct timeval tmp;
	double seconds;

	seconds = ((double)sum->tv_sec / tmstat.used);

	tmp.tv_sec = (uint32_t)seconds;

	seconds -= tmp.tv_sec;

	tmp.tv_usec = (uint32_t)(seconds * 1000000);
	tmp.tv_usec += (sum->tv_usec / tmstat.used);
	
	sum->tv_sec = tmp.tv_sec;
	sum->tv_usec = tmp.tv_usec;
}

/*
 * Perform DNS checks upon the internet address specified by <ip>.
 * 
 * A DNS lookup is done upon the internet address specified by <ip> to get the
 * associated hostname. In case this lookup fails a string representation of
 * the internet address will be stored into the buffer pointed to by <namebuf>
 * not exceeding <maxsize> bytes including the terminating NUL character, and a
 * return value of DNS_CHK_NONE is returned.
 * If the lookup has been successfull, the first hostname in the list of found
 * hostnames is stored to the buffer pointed to by <namebuf> not exceeding
 * <maxsize> bytes including the terminating NUL character. Then another DNS
 * lookup is done upon the hostname to get the internet address for it. If the
 * lookup fails (i.e. the hostname does not map back to anything), a return value
 * of DNS_CHK_FWD is returned. Upon successfull lookup the returned internet
 * address is compared to the internet address specified by <ip>. If both adresses
 * are equal a return value of DNS_CHK_STRICT is returned, otherwise a return
 * value of DNS_CHK_REV is returned.
 */
   
uint8_t dns_check(struct in_addr *ip, char *namebuf, size_t maxsize)
{
	uint8_t lvl = DNS_CHK_NONE;
	struct	hostent *ht;
	char **p;

	/*
	 * Do a forward DNS lookup to get the name for the IP address.
	 */

	if((ht = gethostbyaddr((char *)ip, sizeof(ip), AF_INET)))
	{
		/*
		 * Found DNS name for address, copy it in.
		 */

		sncopy(ht->h_name, namebuf, maxsize);

		lvl++;

		/*
		 * Now check if the name maps back to an address.
		 */

		if((ht = gethostbyname(namebuf)))
		{
			/*
			 * Reverse DNS mapping present, check for strict.
			 */

			lvl++;

			for(p = ht->h_addr_list; *p != NULL; p++)
			{
				if(!bcmp(*p, ip, sizeof(ip)))
				{
					lvl++;
					break;
				}
			}
		}
	}
	else
	{
		/*
		 * No name for this IP address found.
		 * Copy the string representation of the address.
		 */

		sncopy(inet_ntoa(*ip), namebuf, maxsize);
	}

	return lvl;
}

/*
 * Get a string representation of a DNS check level.
 */

char *dnslvlname(uint8_t lvl)
{
	switch(lvl)
	{
		case DNS_CHK_NONE:	return "none";
		case DNS_CHK_FWD:	return "forward";
		case DNS_CHK_REV:	return "reverse";
		case DNS_CHK_STRICT:	return "strict";
	}

	return "<invalid>";
}

/*
 * Parse the DNS check levelname supplied in <arg> and return
 * the numeric value for the specified DNS check level. If the
 * supplied <arg> could not be parsed, a value of 0xFF is returned.
 */

uint8_t dnslvlparse(char *arg)
{
	uint8_t lvl = 0xFF;
	size_t arglen = (arg ? strlen(arg) : 0);

	upperstr(arg);

	if(!arglen)
		return lvl;

	if(!strncmp(arg, "NONE", arglen))
		lvl = DNS_CHK_NONE;
	else if(!strncmp(arg, "FWD", arglen) || !strncmp(arg, "FORWARD", arglen))
		lvl = DNS_CHK_FWD;
	else if(!strncmp(arg, "REVERSE", arglen))
		lvl = DNS_CHK_REV;
	else if(!strncmp(arg, "STRICT", arglen))
		lvl = DNS_CHK_STRICT;

	return lvl;
}
